@@ -3,7 +3,7 @@
 module.exports = {
   overrides: [
     {
-      files: ["imcontact.xml", "imconv.xml", "imconversation.xml"],
+      files: ["imconversation.xml"],
       globals: {
         AppConstants: true,
         chatHandler: true,
