@@ -2,11 +2,11 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this file,
  * You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-/* global MozElements */
+/* global MozElements MozXULElement */
 /* import-globals-from ../../../../../toolkit/content/globalOverlay.js */
 
-// This file is loaded in messenger.xul.
-/* globals fixIterator, MailToolboxCustomizeDone, Notifications, openIMAccountMgr,
+// This file is loaded in messenger.xhtml.
+/* globals fixIterator, MailToolboxCustomizeDone, openIMAccountMgr,
    PROTO_TREE_VIEW, Services, Status, statusSelector, ZoomManager */
 
 var { Notifications } = ChromeUtils.import(
@@ -90,11 +90,23 @@ function enableInlineSpellCheck(aEnableInlineSpellCheck) {
 }
 
 function buddyListContextMenu(aXulMenu) {
-  this.target = aXulMenu.triggerNode;
+  // Clear the context menu from OTR related entries.
+  OTRUI.removeBuddyContextMenu(document);
+
+  this.target = aXulMenu.triggerNode.closest("richlistitem");
+  if (!this.target) {
+    this.shouldDisplay = false;
+    return;
+  }
+
   this.menu = aXulMenu;
   let localName = this.target.localName;
-  this.onContact = localName == "imcontact";
-  this.onConv = localName == "imconv";
+  this.onContact =
+    localName == "richlistitem" &&
+    this.target.getAttribute("is") == "chat-contact-richlistitem";
+  this.onConv =
+    localName == "richlistitem" &&
+    this.target.getAttribute("is") == "chat-imconv-richlistitem";
   this.shouldDisplay = this.onContact || this.onConv;
 
   let hide = !this.onContact;
@@ -111,8 +123,18 @@ function buddyListContextMenu(aXulMenu) {
   document.getElementById("context-openconversation").disabled =
     !hide && !this.target.canOpenConversation();
 
-  if (gOtrEnabled) {
-    OTRUI.addBuddyContextMenu(this.menu, document);
+  // Show OTR related context menu items if:
+  // - The OTR feature is currently enabled.
+  // - The target's status is not currently offline or unknown.
+  // - The target can send messages.
+  if (
+    gOtrEnabled &&
+    this.target.contact &&
+    this.target.contact.statusType != Ci.imIStatusInfo.STATUS_UNKNOWN &&
+    this.target.contact.statusType != Ci.imIStatusInfo.STATUS_OFFLINE &&
+    this.target.contact.canSendMessage
+  ) {
+    OTRUI.addBuddyContextMenu(this.menu, document, this.target.contact);
   }
 }
 
@@ -292,7 +314,7 @@ var chatTabType = {
     gChatTab = aTab;
     chatHandler._onTabActivated();
     // The next call may change the selected conversation, but that
-    // will be handled by the selected mutation observer of the imconv.
+    // will be handled by the selected mutation observer of the chat-imconv-richlistitem.
     chatHandler._updateSelectedConversation();
     chatHandler._updateFocus();
   },
@@ -374,8 +396,8 @@ var chatTabType = {
       ) {
         return document.getElementById("conv-log-browser");
       }
-    } else if (panel && panel.localName == "imconversation") {
-      return panel.browser;
+    } else if (panel && panel.localName == "chat-conversation") {
+      return panel.convBrowser;
     }
     return null;
   },
@@ -388,7 +410,7 @@ var chatTabType = {
       ) {
         return document.getElementById("log-findbar");
       }
-    } else if (panel && panel.localName == "imconversation") {
+    } else if (panel && panel.localName == "chat-conversation") {
       return panel.findbar;
     }
     return null;
@@ -399,14 +421,14 @@ var chatTabType = {
 
 var chatHandler = {
   get msgNotificationBar() {
-    delete this.msgNotificationBar;
+    delete this._msgNotificationBar;
 
     let newNotificationBox = new MozElements.NotificationBox(element => {
       element.setAttribute("notificationside", "top");
       document.getElementById("chat-notification-top").prepend(element);
     });
 
-    return (this.msgNotificationBar = newNotificationBox);
+    return (this._msgNotificationBar = newNotificationBox);
   },
 
   _addConversation(aConv) {
@@ -417,7 +439,9 @@ var chatHandler = {
       gChatTab &&
       gChatTab.tabNode.selected &&
       (!selectedItem ||
-        (selectedItem == convs && convs.nextSibling.localName != "imconv"));
+        (selectedItem == convs &&
+          convs.nextElementSibling.localName != "richlistitem" &&
+          convs.nextSibling.getAttribute("is") != "chat-imconv-richlistitem"));
     let elt = convs.addContact(aConv, "imconv");
     if (shouldSelect) {
       list.selectedItem = elt;
@@ -466,26 +490,32 @@ var chatHandler = {
       return;
     }
 
-    let [unreadTargettedCount, unreadTotalCount] = this.countUnreadMessages();
-    chatButton.badgeCount = unreadTargettedCount;
+    let [
+      unreadTargettedCount,
+      unreadTotalCount,
+      unreadOTRNotificationCount,
+    ] = this.countUnreadMessages();
+    let unreadMsgAndNotificationCount =
+      unreadTargettedCount + unreadOTRNotificationCount;
+    chatButton.badgeCount = unreadMsgAndNotificationCount;
 
-    if (unreadTotalCount) {
+    if (unreadTotalCount || unreadOTRNotificationCount) {
       chatButton.setAttribute("unreadMessages", "true");
     } else {
       chatButton.removeAttribute("unreadMessages");
     }
 
-    if (unreadTargettedCount != this._notifiedUnreadCount) {
+    if (unreadMsgAndNotificationCount != this._notifiedUnreadCount) {
       let unreadInt = Cc["@mozilla.org/supports-PRInt32;1"].createInstance(
         Ci.nsISupportsPRInt32
       );
-      unreadInt.data = unreadTargettedCount;
+      unreadInt.data = unreadMsgAndNotificationCount;
       Services.obs.notifyObservers(
         unreadInt,
         "unread-im-count-changed",
-        unreadTargettedCount
+        unreadMsgAndNotificationCount
       );
-      this._notifiedUnreadCount = unreadTargettedCount;
+      this._notifiedUnreadCount = unreadMsgAndNotificationCount;
     }
   },
 
@@ -493,11 +523,13 @@ var chatHandler = {
     let convs = imServices.conversations.getUIConversations();
     let unreadTargettedCount = 0;
     let unreadTotalCount = 0;
+    let unreadOTRNotificationCount = 0;
     for (let conv of convs) {
       unreadTargettedCount += conv.unreadTargetedMessageCount;
       unreadTotalCount += conv.unreadIncomingMessageCount;
+      unreadOTRNotificationCount += conv.unreadOTRNotificationCount;
     }
-    return [unreadTargettedCount, unreadTotalCount];
+    return [unreadTargettedCount, unreadTotalCount, unreadOTRNotificationCount];
   },
 
   updateTitle() {
@@ -513,7 +545,8 @@ var chatHandler = {
       let selectedItem = document.getElementById("contactlistbox").selectedItem;
       if (
         selectedItem &&
-        selectedItem.localName == "imconv" &&
+        selectedItem.localName == "richlistitem" &&
+        selectedItem.getAttribute("is") == "chat-imconv-richlistitem" &&
         !selectedItem.hidden
       ) {
         title += " - " + selectedItem.getAttribute("displayname");
@@ -526,7 +559,7 @@ var chatHandler = {
   onConvResize() {
     let convDeck = document.getElementById("conversationsDeck");
     let panel = convDeck.selectedPanel;
-    if (panel && panel.localName == "imconversation") {
+    if (panel && panel.localName == "chat-conversation") {
       panel.onConvResize();
     }
   },
@@ -583,9 +616,7 @@ var chatHandler = {
     cti.setAttribute("displayName", aConversation.title);
 
     // Find and display the contact for this log.
-    let accounts = imServices.accounts.getAccounts();
-    while (accounts.hasMoreElements()) {
-      let account = accounts.getNext();
+    for (let account of imServices.accounts.getAccounts()) {
       if (
         account.normalizedName == aConversation.account.normalizedName &&
         account.protocol.normalizedName == aConversation.account.protocol.name
@@ -737,12 +768,18 @@ var chatHandler = {
     if (!item) {
       return;
     }
-    if (item.localName == "imconv") {
+    if (
+      item.localName == "richlistitem" &&
+      item.getAttribute("is") == "chat-imconv-richlistitem"
+    ) {
       document.getElementById("conversationsDeck").selectedPanel =
         item.convView;
       document.getElementById("logTree").view.selection.clearSelection();
       item.convView.focus();
-    } else if (item.localName == "imcontact") {
+    } else if (
+      item.localName == "richlistitem" &&
+      item.getAttribute("is") == "chat-contact-richlistitem"
+    ) {
       item.openConversation();
     }
   },
@@ -793,14 +830,23 @@ var chatHandler = {
       return;
     }
     let item = document.getElementById("contactlistbox").selectedItem;
-    if (item.localName == "imconv" && item.convView) {
+    if (
+      item.localName == "richlistitem" &&
+      item.getAttribute("is") == "chat-imconv-richlistitem" &&
+      item.convView
+    ) {
       item.convView.focus();
     }
   },
   onListItemSelected() {
     let contactlistbox = document.getElementById("contactlistbox");
     let item = contactlistbox.selectedItem;
-    if (!item || item.hidden || item.localName == "imgroup") {
+    if (
+      !item ||
+      item.hidden ||
+      (item.localName == "richlistitem" &&
+        item.getAttribute("is") == "chat-group-richlistitem")
+    ) {
       this._hideContextPane(true);
       document.getElementById(
         "conversationsDeck"
@@ -844,16 +890,18 @@ var chatHandler = {
           this._showLogList(aSimilarLogs, aLog);
         });
       });
-    } else if (item.localName == "imconv") {
+    } else if (
+      item.localName == "richlistitem" &&
+      item.getAttribute("is") == "chat-imconv-richlistitem"
+    ) {
       let convDeck = document.getElementById("conversationsDeck");
       if (!item.convView) {
-        // Create new conversation binding.
-        let conv = document.createXULElement("imconversation");
+        let conv = document.createXULElement("chat-conversation");
         convDeck.appendChild(conv);
         conv.conv = item.conv;
         conv.tab = item;
-        conv.setAttribute("contentcontextmenu", "chatConversationContextMenu");
-        conv.setAttribute("contenttooltip", "imTooltip");
+        conv.setAttribute("contextmenu", "chatConversationContextMenu");
+        conv.setAttribute("tooltip", "imTooltip");
         item.convView = conv;
         document.getElementById("contextSplitter").hidden = false;
         document.getElementById("contextPane").hidden = false;
@@ -869,7 +917,7 @@ var chatHandler = {
         // Set "mail editor mask" so changing the language doesn't
         // affect the global preference and multiple chats can have
         // individual languages.
-        conv.editor.editor.flags |= Ci.nsIPlaintextEditor.eEditorMailMask;
+        conv.editor.editor.flags |= Ci.nsIEditor.eEditorMailMask;
 
         // Initialise language to the default.
         conv.editor.setAttribute(
@@ -901,12 +949,13 @@ var chatHandler = {
         this._showLogList(aLogs);
       });
 
-      let contextPane = document.getElementById("contextPane");
+      document
+        .querySelectorAll("#contextPaneFlexibleBox .conv-chat")
+        .forEach(e => {
+          e.setAttribute("hidden", !item.conv.isChat);
+        });
       if (item.conv.isChat) {
-        contextPane.setAttribute("chat", "true");
         item.convView.showParticipants();
-      } else {
-        contextPane.removeAttribute("chat");
       }
 
       let button = document.getElementById("goToConversation");
@@ -914,7 +963,10 @@ var chatHandler = {
       button.label = bundle.getString("goBackToCurrentConversation.button");
       button.disabled = false;
       this.observedContact = null;
-    } else if (item.localName == "imcontact") {
+    } else if (
+      item.localName == "richlistitem" &&
+      item.getAttribute("is") == "chat-contact-richlistitem"
+    ) {
       if (gOtrEnabled) {
         OTRUI.hideOTRButton();
       }
@@ -925,14 +977,18 @@ var chatHandler = {
         this.observedContact.id == contact.id
       ) {
         return; // onselect has just been fired again because a status
-        // change caused the imcontact to move.
+        // change caused the chat-contact-richlistitem to move.
         // Return early to avoid flickering and changing the selected log.
       }
 
       this.showContactInfo(contact);
       this.observedContact = contact;
 
-      document.getElementById("contextPane").removeAttribute("chat");
+      document
+        .querySelectorAll("#contextPaneFlexibleBox .conv-chat")
+        .forEach(e => {
+          e.setAttribute("hidden", "true");
+        });
 
       imServices.logs.getLogsForContact(contact, true).then(aLogs => {
         if (contactlistbox.selectedItem != item) {
@@ -994,7 +1050,7 @@ var chatHandler = {
   _openDialog(aType) {
     let features = "chrome,modal,titlebar,centerscreen";
     window.openDialog(
-      "chrome://messenger/content/chat/" + aType + ".xul",
+      "chrome://messenger/content/chat/" + aType + ".xhtml",
       "",
       features
     );
@@ -1007,7 +1063,7 @@ var chatHandler = {
   },
 
   _colorCache: {},
-  // Duplicated code from imconversation.xml :-(
+  // Duplicated code from chat-conversation.js :-(
   _computeColor(aName) {
     if (Object.prototype.hasOwnProperty.call(this._colorCache, aName)) {
       return this._colorCache[aName];
@@ -1042,7 +1098,7 @@ var chatHandler = {
     let connected = false;
     let hasAccount = false;
     let canJoinChat = false;
-    for (let account of fixIterator(imServices.accounts.getAccounts())) {
+    for (let account of imServices.accounts.getAccounts()) {
       hasAccount = true;
       if (account.connected) {
         connected = true;
@@ -1063,6 +1119,7 @@ var chatHandler = {
         ? "openIMAccountManagerButton"
         : "openIMAccountWizardButton";
     }
+
     for (let id of [
       "statusTypeIcon",
       "statusMessage",
@@ -1073,26 +1130,44 @@ var chatHandler = {
         elt.disabled = !hasAccount;
       }
     }
-    for (let id of [
-      "button-add-buddy",
-      "newIMContactMenuItem",
-      "appmenu_newIMContactMenuItem",
-    ]) {
-      let elt = document.getElementById(id);
-      if (elt) {
-        elt.disabled = !connected;
+
+    let chatStatusCmd = document.getElementById("cmd_chatStatus");
+    if (chatStatusCmd) {
+      if (hasAccount) {
+        chatStatusCmd.removeAttribute("disabled");
+      } else {
+        chatStatusCmd.setAttribute("disabled", true);
       }
     }
-    for (let id of [
-      "button-join-chat",
-      "joinChatMenuItem",
-      "appmenu_joinChatMenuItem",
-    ]) {
-      let elt = document.getElementById(id);
-      if (elt) {
-        elt.disabled = !canJoinChat;
+
+    let addBuddyButton = document.getElementById("button-add-buddy");
+    if (addBuddyButton) {
+      addBuddyButton.disabled = !connected;
+    }
+
+    let addBuddyCmd = document.getElementById("cmd_addChatBuddy");
+    if (addBuddyCmd) {
+      if (connected) {
+        addBuddyCmd.removeAttribute("disabled");
+      } else {
+        addBuddyCmd.setAttribute("disabled", true);
       }
     }
+
+    let joinChatButton = document.getElementById("button-join-chat");
+    if (joinChatButton) {
+      joinChatButton.disabled = !canJoinChat;
+    }
+
+    let joinChatCmd = document.getElementById("cmd_joinChat");
+    if (joinChatCmd) {
+      if (canJoinChat) {
+        joinChatCmd.removeAttribute("disabled");
+      } else {
+        joinChatCmd.setAttribute("disabled", true);
+      }
+    }
+
     let groupIds = ["conversations", "onlinecontacts", "offlinecontacts"];
     let contactlist = document.getElementById("contactlistbox");
     if (
@@ -1119,7 +1194,8 @@ var chatHandler = {
     let selectedItem = list.selectedItem;
     if (
       selectedItem &&
-      selectedItem.localName == "imconv" &&
+      selectedItem.localName == "richlistitem" &&
+      selectedItem.getAttribute("is") == "chat-imconv-richlistitem" &&
       selectedItem.directedUnreadCount
     ) {
       selectedItem.update();
@@ -1128,7 +1204,7 @@ var chatHandler = {
 
     let firstConv;
     let convs = document.getElementById("conversationsGroup");
-    let conv = convs.nextSibling;
+    let conv = convs.nextElementSibling;
     while (conv.id != "searchResultConv") {
       if (!firstConv) {
         firstConv = conv;
@@ -1138,13 +1214,17 @@ var chatHandler = {
         list.selectedItem = conv;
         return;
       }
-      conv = conv.nextSibling;
+      conv = conv.nextElementSibling;
     }
 
     // No unread messages, select the first conversation, but only if
     // the existing selection is uninteresting (a section header).
     if (firstConv) {
-      if (!selectedItem || selectedItem.localName == "imgroup") {
+      if (
+        !selectedItem ||
+        (selectedItem.localName == "richlistitem" &&
+          selectedItem.getAttribute("is") == "chat-group-richlistitem")
+      ) {
         list.selectedItem = firstConv;
       }
       return;
@@ -1176,7 +1256,11 @@ var chatHandler = {
       return null;
     }
     let selectedItem = list.selectedItem;
-    if (!selectedItem || selectedItem.localName != "imconv") {
+    if (
+      !selectedItem ||
+      (selectedItem.localName != "richlistitem" &&
+        selectedItem.getAttribute("is") != "chat-imconv-richlistitem")
+    ) {
       return null;
     }
     let convView = selectedItem.convView;
@@ -1436,7 +1520,10 @@ var chatHandler = {
 
     let rawConversations = conversations.map(c => c.conv);
     let current;
-    if (aList.selectedItem.localName === "imconv") {
+    if (
+      aList.selectedItem.localName == "richlistitem" &&
+      aList.selectedItem.getAttribute("is") == "chat-imconv-richlistitem"
+    ) {
       current = aList.selectedIndex - aList.getIndexOfItem(conversations[0]);
     }
     let newIndex = this._getNextUnreadConversation(
@@ -1512,7 +1599,6 @@ var chatHandler = {
           return;
         }
         listbox._userSelecting = true;
-
         let reverse = aEvent.keyCode != aEvent.DOM_VK_DOWN;
         if (aEvent.shiftKey) {
           chatHandler._selectNextUnreadConversation(reverse, listbox);
@@ -1521,7 +1607,11 @@ var chatHandler = {
         }
         listbox._userSelecting = false;
         let item = listbox.selectedItem;
-        if (item.localName == "imconv" && item.convView) {
+        if (
+          item.localName == "richlistitem" &&
+          item.getAttribute("is") == "chat-imconv-richlistitem" &&
+          item.convView
+        ) {
           item.convView.focus();
         } else {
           listbox.focus();
@@ -1696,8 +1786,7 @@ chatLogTreeView.prototype = {
 
     // Build a chatLogTreeLogItem for each log, and put it in the right group.
     let groups = {};
-    while (this._logs.hasMoreElements()) {
-      let log = this._logs.getNext();
+    for (let log of this._logs) {
       let logDate = new Date(log.time * 1000);
       // Calculate elapsed time between the log and 00:00:00 today.
       let timeFromToday = todayDate - logDate;
