@@ -172,7 +172,7 @@ var gAccountManager = {
     }
 
     // The following notification handlers need an account.
-    aObject.QueryInterface(Ci.imIAccount);
+    let account = aObject.QueryInterface(Ci.imIAccount);
 
     if (aTopic == "account-added") {
       document.getElementById("accountsDesk").selectedIndex = 1;
@@ -180,12 +180,12 @@ var gAccountManager = {
         is: "chat-account-richlistitem",
       });
       this.accountList.appendChild(elt);
-      elt.build(aObject);
+      elt.build(account);
       if (this.accountList.getRowCount() == 1) {
         this.accountList.selectedIndex = 0;
       }
     } else if (aTopic == "account-removed") {
-      let elt = document.getElementById(aObject.id);
+      let elt = document.getElementById(account.id);
       elt.destroy();
       if (!elt.selected) {
         elt.remove();
@@ -208,12 +208,35 @@ var gAccountManager = {
       }
       this.accountList.selectedIndex = selectedIndex;
     } else if (aTopic == "account-updated") {
-      document.getElementById(aObject.id).build(aObject);
+      document.getElementById(account.id).build(account);
       this.disableCommandItems();
     } else if (aTopic == "account-connect-progress") {
-      document.getElementById(aObject.id).updateConnectionState();
+      document.getElementById(account.id).updateConnectionState();
     } else if (aTopic == "account-connect-error") {
-      document.getElementById(aObject.id).updateConnectionError();
+      document.getElementById(account.id).updateConnectionError();
+      // See NSSErrorsService::ErrorIsOverridable.
+      if (
+        [
+          "MOZILLA_PKIX_ERROR_ADDITIONAL_POLICY_CONSTRAINT_FAILED",
+          "MOZILLA_PKIX_ERROR_CA_CERT_USED_AS_END_ENTITY",
+          "MOZILLA_PKIX_ERROR_EMPTY_ISSUER_NAME",
+          "MOZILLA_PKIX_ERROR_INADEQUATE_KEY_SIZE",
+          "MOZILLA_PKIX_ERROR_MITM_DETECTED",
+          "MOZILLA_PKIX_ERROR_NOT_YET_VALID_CERTIFICATE",
+          "MOZILLA_PKIX_ERROR_NOT_YET_VALID_ISSUER_CERTIFICATE",
+          "MOZILLA_PKIX_ERROR_SELF_SIGNED_CERT",
+          "MOZILLA_PKIX_ERROR_V1_CERT_USED_AS_CA",
+          "SEC_ERROR_CA_CERT_INVALID",
+          "SEC_ERROR_CERT_SIGNATURE_ALGORITHM_DISABLED",
+          "SEC_ERROR_EXPIRED_CERTIFICATE",
+          "SEC_ERROR_EXPIRED_ISSUER_CERTIFICATE",
+          "SEC_ERROR_INVALID_TIME",
+          "SEC_ERROR_UNKNOWN_ISSUER",
+          "SSL_ERROR_BAD_CERT_DOMAIN",
+        ].includes(account.prplAccount.securityInfo?.errorCodeString)
+      ) {
+        this.addException();
+      }
     } else {
       const stateEvents = {
         "account-connected": "connected",
@@ -222,7 +245,7 @@ var gAccountManager = {
         "account-disconnecting": "disconnecting",
       };
       if (aTopic in stateEvents) {
-        let elt = document.getElementById(aObject.id);
+        let elt = document.getElementById(account.id);
         if (!elt) {
           // Probably disconnecting a removed account.
           return;
@@ -259,14 +282,14 @@ var gAccountManager = {
   addException() {
     let account = this.accountList.selectedItem.account;
     let prplAccount = account.prplAccount;
-    if (!account.disconnected || !prplAccount.connectionTarget) {
+    if (!prplAccount.connectionTarget) {
       return;
     }
 
     // Open the Gecko SSL exception dialog.
     let params = {
       exceptionAdded: false,
-      securityInfo: prplAccount.secInfo,
+      securityInfo: prplAccount.securityInfo,
       prefetchCert: true,
       location: prplAccount.connectionTarget,
     };
@@ -278,6 +301,7 @@ var gAccountManager = {
     );
     // Reconnect the account if an exception was added.
     if (params.exceptionAdded) {
+      account.disconnect();
       account.connect();
     }
   },
