@@ -144,7 +144,7 @@ function OpenBrowserWindow()
     // current charset menu setting from the current document
     // and use it to initialize the new browser window
     return window.openDialog(getBrowserURL(), "_blank",
-                             "chrome,all,dialog=no", null,
+                             "chrome,all,dialog=no,non-private", null,
                              "charset=" + window.content.document.characterSet);
   }
 
@@ -152,7 +152,7 @@ function OpenBrowserWindow()
     // if a browser window already exists then set startpage to null so
     // navigator.js can check pref for how new window should be opened
     return win.openDialog(getBrowserURL(), "_blank",
-                          "chrome,all,dialog=no", null);
+                          "chrome,all,dialog=no,non-private", null);
   }
 
   // open the first browser window as if we were starting up
@@ -172,29 +172,36 @@ function OpenBrowserWindow()
   return null;
 }
 
-function CycleWindow( aType )
-{
-  var topWindowOfType = Services.wm.getMostRecentWindow(aType);
-  var topWindow = Services.wm.getMostRecentWindow(null);
-
-  if ( topWindowOfType == null )
+function CycleWindow(aType) {
+  let topWindowOfType = Services.wm.getMostRecentWindow(aType);
+  if (topWindowOfType == null)
     return null;
 
-  if ( topWindowOfType != topWindow ) {
+  let topWindow = Services.wm.getMostRecentWindow(null);
+  if (topWindowOfType != topWindow) {
     toOpenWindow(topWindowOfType);
     return topWindowOfType;
   }
 
-  var enumerator = Services.wm.getEnumerator(aType);
-  var firstWindow = enumerator.getNext();
-  var iWindow = firstWindow;
-  while (iWindow != topWindow && enumerator.hasMoreElements())
-    iWindow = enumerator.getNext();
+  let topFound = false;
+  let enumerator = Services.wm.getEnumerator(aType);
+  let iWindow;
+  let firstWindow;
 
-  if (enumerator.hasMoreElements()) {
+  while (enumerator.hasMoreElements()) {
     iWindow = enumerator.getNext();
-    toOpenWindow(iWindow);
-    return iWindow;
+    if (!iWindow.closed) {
+      if (!firstWindow) {
+        firstWindow = iWindow;
+      }
+      if (topFound) {
+        toOpenWindow(iWindow);
+        return iWindow;
+      }
+      if (iWindow == topWindow) {
+        topFound = true;
+      }
+    }
   }
 
   if (firstWindow == topWindow) // Only one window
@@ -219,7 +226,7 @@ function checkFocusedWindow()
   let frag = document.createDocumentFragment();
   while (windows.hasMoreElements()) {
     let win = windows.getNext();
-    if (win.document.documentElement.getAttribute("inwindowmenu") == "false") {
+    if (win.closed || win.document.documentElement.getAttribute("inwindowmenu") == "false") {
       continue;
     }
     let item = document.createElement("menuitem");
@@ -259,7 +266,7 @@ function toProfileManager()
   // that was returned in the param block.
 }
 
-// This function is used by mac's platformCommunicatorOverlay
+// This function is only used by macs.
 function ZoomCurrentWindow()
 {
   if (window.windowState == STATE_NORMAL)
