@@ -1,5 +1,3 @@
-/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
-/* vim: set ft=javascript ts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -11,12 +9,9 @@ var Services = require("Services");
 var defer = require("devtools/shared/defer");
 var DevToolsUtils = require("devtools/shared/DevToolsUtils");
 var { dumpn, dumpv } = DevToolsUtils;
-loader.lazyRequireGetter(this, "prompt",
-  "devtools/shared/security/prompt");
-loader.lazyRequireGetter(this, "cert",
-  "devtools/shared/security/cert");
-loader.lazyRequireGetter(this, "asyncStorage",
-  "devtools/shared/async-storage");
+loader.lazyRequireGetter(this, "prompt", "devtools/shared/security/prompt");
+loader.lazyRequireGetter(this, "cert", "devtools/shared/security/cert");
+loader.lazyRequireGetter(this, "asyncStorage", "devtools/shared/async-storage");
 
 /**
  * A simple enum-like object with keys mirrored to values.
@@ -24,7 +19,7 @@ loader.lazyRequireGetter(this, "asyncStorage",
  * mis-type the value.
  */
 function createEnum(obj) {
-  for (let key in obj) {
+  for (const key in obj) {
     obj[key] = key;
   }
   return obj;
@@ -36,8 +31,7 @@ function createEnum(obj) {
  * centralize the common actions available, while still allowing embedders to
  * present their UI in whatever way they choose.
  */
-var AuthenticationResult = exports.AuthenticationResult = createEnum({
-
+var AuthenticationResult = (exports.AuthenticationResult = createEnum({
   /**
    * Close all listening sockets, and disable them from opening again.
    */
@@ -63,9 +57,8 @@ var AuthenticationResult = exports.AuthenticationResult = createEnum({
    * connections from the same client.  This requires a trustable mechanism to
    * identify the client in the future, such as the cert used during OOB_CERT.
    */
-  ALLOW_PERSIST: null
-
-});
+  ALLOW_PERSIST: null,
+}));
 
 /**
  * An |Authenticator| implements an authentication mechanism via various hooks
@@ -86,13 +79,12 @@ var Authenticators = {};
  * no cryptographic properties at work here, so it is up to the user to be sure
  * that the client can be trusted.
  */
-var Prompt = Authenticators.Prompt = {};
+var Prompt = (Authenticators.Prompt = {});
 
 Prompt.mode = "PROMPT";
 
-Prompt.Client = function () {};
+Prompt.Client = function() {};
 Prompt.Client.prototype = {
-
   mode: Prompt.mode,
 
   /**
@@ -107,9 +99,9 @@ Prompt.Client.prototype = {
    * to ensure it meets the authenticator's policies.
    *
    * @param host string
-   *        The host name or IP address of the debugger server.
+   *        The host name or IP address of the devtools server.
    * @param port number
-   *        The port number of the debugger server.
+   *        The port number of the devtools server.
    * @param encryption boolean (optional)
    *        Whether the server requires encryption.  Defaults to false.
    * @param cert object (optional)
@@ -130,9 +122,9 @@ Prompt.Client.prototype = {
    * Debugging commences after this hook completes successfully.
    *
    * @param host string
-   *        The host name or IP address of the debugger server.
+   *        The host name or IP address of the devtools server.
    * @param port number
-   *        The port number of the debugger server.
+   *        The port number of the devtools server.
    * @param encryption boolean (optional)
    *        Whether the server requires encryption.  Defaults to false.
    * @param transport DebuggerTransport
@@ -140,12 +132,10 @@ Prompt.Client.prototype = {
    * @return A promise can be used if there is async behavior.
    */
   authenticate() {},
-
 };
 
-Prompt.Server = function () {};
+Prompt.Server = function() {};
 Prompt.Server.prototype = {
-
   mode: Prompt.mode,
 
   /**
@@ -207,7 +197,7 @@ Prompt.Server.prototype = {
     return this.allowConnection({
       authentication: this.mode,
       client,
-      server
+      server,
     });
   },
 
@@ -236,7 +226,6 @@ Prompt.Server.prototype = {
    *         A promise that will be resolved to the above is also allowed.
    */
   allowConnection: prompt.Server.defaultAllowConnection,
-
 };
 
 /**
@@ -257,13 +246,12 @@ Prompt.Server.prototype = {
  *
  * See docs/wifi.md for details of the authentication design.
  */
-var OOBCert = Authenticators.OOBCert = {};
+var OOBCert = (Authenticators.OOBCert = {});
 
 OOBCert.mode = "OOB_CERT";
 
-OOBCert.Client = function () {};
+OOBCert.Client = function() {};
 OOBCert.Client.prototype = {
-
   mode: OOBCert.mode,
 
   /**
@@ -282,9 +270,9 @@ OOBCert.Client.prototype = {
    * to ensure it meets the authenticator's policies.
    *
    * @param host string
-   *        The host name or IP address of the debugger server.
+   *        The host name or IP address of the devtools server.
    * @param port number
-   *        The port number of the debugger server.
+   *        The port number of the devtools server.
    * @param encryption boolean (optional)
    *        Whether the server requires encryption.  Defaults to false.
    * @param cert object (optional)
@@ -294,14 +282,16 @@ OOBCert.Client.prototype = {
    * @return boolean
    *         Whether the connection is valid.
    */
+  // eslint-disable-next-line no-shadow
   validateConnection({ cert, socket }) {
     // Step B.7
     // Client verifies that Server's cert matches hash(ServerCert) from the
     // advertisement
     dumpv("Validate server cert hash");
-    let serverCert = socket.securityInfo.QueryInterface(Ci.nsISSLStatusProvider)
-                           .SSLStatus.serverCert;
-    let advertisedCert = cert;
+    const serverCert = socket.securityInfo.QueryInterface(
+      Ci.nsITransportSecurityInfo
+    ).serverCert;
+    const advertisedCert = cert;
     if (serverCert.sha256Fingerprint != advertisedCert.sha256) {
       dumpn("Server cert hash doesn't match advertisement");
       return false;
@@ -316,9 +306,9 @@ OOBCert.Client.prototype = {
    * Debugging commences after this hook completes successfully.
    *
    * @param host string
-   *        The host name or IP address of the debugger server.
+   *        The host name or IP address of the devtools server.
    * @param port number
-   *        The port number of the debugger server.
+   *        The port number of the devtools server.
    * @param encryption boolean (optional)
    *        Whether the server requires encryption.  Defaults to false.
    * @param cert object (optional)
@@ -327,24 +317,25 @@ OOBCert.Client.prototype = {
    *        A transport that can be used to communicate with the server.
    * @return A promise can be used if there is async behavior.
    */
+  // eslint-disable-next-line no-shadow
   authenticate({ host, port, cert, transport }) {
-    let deferred = defer();
+    const deferred = defer();
     let oobData;
 
     let activeSendDialog;
-    let closeDialog = () => {
+    const closeDialog = () => {
       // Close any prompts the client may have been showing from previous
       // authentication steps
-      if (activeSendDialog && activeSendDialog.close) {
+      if (activeSendDialog?.close) {
         activeSendDialog.close();
         activeSendDialog = null;
       }
     };
 
     transport.hooks = {
-      onPacket: async (packet) => {
+      onPacket: async packet => {
         closeDialog();
-        let { authResult } = packet;
+        const { authResult } = packet;
         switch (authResult) {
           case AuthenticationResult.PENDING:
             // Step B.8
@@ -355,7 +346,7 @@ OOBCert.Client.prototype = {
               port,
               cert,
               authResult,
-              oob: oobData
+              oob: oobData,
             });
             break;
           case AuthenticationResult.ALLOW:
@@ -387,7 +378,7 @@ OOBCert.Client.prototype = {
         // Transport died before auth completed
         transport.hooks = null;
         deferred.reject(reason);
-      }
+      },
     };
     transport.ready();
     return deferred.promise;
@@ -398,19 +389,20 @@ OOBCert.Client.prototype = {
    * channel.
    */
   async _createOOB() {
-    let clientCert = await cert.local.getOrCreate();
+    const clientCert = await cert.local.getOrCreate();
     return {
       sha256: clientCert.sha256Fingerprint,
-      k: this._createRandom()
+      k: this._createRandom(),
     };
   },
 
   _createRandom() {
     // 16 bytes / 128 bits
     const length = 16;
-    let rng = Cc["@mozilla.org/security/random-generator;1"]
-              .createInstance(Ci.nsIRandomGenerator);
-    let bytes = rng.generateRandomBytes(length);
+    const rng = Cc["@mozilla.org/security/random-generator;1"].createInstance(
+      Ci.nsIRandomGenerator
+    );
+    const bytes = rng.generateRandomBytes(length);
     return bytes.map(byte => byte.toString(16)).join("");
   },
 
@@ -418,9 +410,9 @@ OOBCert.Client.prototype = {
    * Send data across the OOB channel to the server to authenticate the devices.
    *
    * @param host string
-   *        The host name or IP address of the debugger server.
+   *        The host name or IP address of the devtools server.
    * @param port number
-   *        The port number of the debugger server.
+   *        The port number of the devtools server.
    * @param cert object (optional)
    *        The server's cert details.
    * @param authResult AuthenticationResult
@@ -433,12 +425,10 @@ OOBCert.Client.prototype = {
    *         * close: Function to hide the notification
    */
   sendOOB: prompt.Client.defaultSendOOB,
-
 };
 
-OOBCert.Server = function () {};
+OOBCert.Server = function() {};
 OOBCert.Server.prototype = {
-
   mode: OOBCert.mode,
 
   /**
@@ -463,7 +453,7 @@ OOBCert.Server.prototype = {
    *        The socket that is about to start listening.
    */
   augmentSocketOptions(listener, socket) {
-    let requestCert = Ci.nsITLSServerSocket.REQUIRE_ALWAYS;
+    const requestCert = Ci.nsITLSServerSocket.REQUIRE_ALWAYS;
     socket.setRequestClientCertificate(requestCert);
   },
 
@@ -514,11 +504,11 @@ OOBCert.Server.prototype = {
     // Step B.3 / C.3
     // TLS connection established, authentication begins
     const storageKey = `devtools.auth.${this.mode}.approved-clients`;
-    let approvedClients = (await asyncStorage.getItem(storageKey)) || {};
+    const approvedClients = (await asyncStorage.getItem(storageKey)) || {};
     // Step C.4
     // Server sees that ClientCert is from a known client via hash(ClientCert)
     if (approvedClients[client.cert.sha256]) {
-      let authResult = AuthenticationResult.ALLOW_PERSIST;
+      const authResult = AuthenticationResult.ALLOW_PERSIST;
       transport.send({ authResult });
       // Step C.5
       // Debugging begins
@@ -529,16 +519,16 @@ OOBCert.Server.prototype = {
     // Server sees that ClientCert is from a unknown client
     // Tell client they are unknown and should display OOB client UX
     transport.send({
-      authResult: AuthenticationResult.PENDING
+      authResult: AuthenticationResult.PENDING,
     });
 
     // Step B.5
     // User is shown a Allow / Deny / Always Allow prompt on the Server
     // with Client name and hash(ClientCert)
-    let authResult = await this.allowConnection({
+    const authResult = await this.allowConnection({
       authentication: this.mode,
       client,
-      server
+      server,
     });
 
     switch (authResult) {
@@ -552,13 +542,13 @@ OOBCert.Server.prototype = {
     }
 
     // Examine additional data for authentication
-    let oob = await this.receiveOOB();
+    const oob = await this.receiveOOB();
     if (!oob) {
       dumpn("Invalid OOB data received");
       return AuthenticationResult.DENY;
     }
 
-    let { sha256, k } = oob;
+    const { sha256, k } = oob;
     // The OOB auth prompt should have transferred:
     // hash(ClientCert) + K(random 128-bit number)
     // from the client.
@@ -635,7 +625,6 @@ OOBCert.Server.prototype = {
    *         A promise that will be resolved to the above is also allowed.
    */
   receiveOOB: prompt.Server.defaultReceiveOOB,
-
 };
 
 exports.Authenticators = {
@@ -643,12 +632,12 @@ exports.Authenticators = {
     if (!mode) {
       mode = Prompt.mode;
     }
-    for (let key in Authenticators) {
-      let auth = Authenticators[key];
+    for (const key in Authenticators) {
+      const auth = Authenticators[key];
       if (auth.mode === mode) {
         return auth;
       }
     }
     throw new Error("Unknown authenticator mode: " + mode);
-  }
+  },
 };
