@@ -9,10 +9,9 @@
 #include "nsURLHelper.h"
 #include "nsNetCID.h"
 #include "nsMimeTypes.h"
-#include "nsIContentSniffer.h"
+#include "nsUnknownDecoder.h"
 #include "nsIScriptSecurityManager.h"
 #include "nsMimeTypes.h"
-#include "nsIHttpChannel.h"
 #include "nsIChannelEventSink.h"
 #include "nsIStreamConverterService.h"
 #include "nsChannelClassifier.h"
@@ -23,11 +22,14 @@
 #include "LoadInfo.h"
 #include "nsServiceManagerUtils.h"
 #include "nsRedirectHistoryEntry.h"
+#include "mozilla/BasePrincipal.h"
+
+using namespace mozilla;
 
 // This class is used to suspend a request across a function scope.
 class ScopedRequestSuspender {
  public:
-  explicit ScopedRequestSuspender(nsIRequest *request) : mRequest(request) {
+  explicit ScopedRequestSuspender(nsIRequest* request) : mRequest(request) {
     if (mRequest && NS_FAILED(mRequest->Suspend())) {
       NS_WARNING("Couldn't suspend pump");
       mRequest = nullptr;
@@ -38,7 +40,7 @@ class ScopedRequestSuspender {
   }
 
  private:
-  nsIRequest *mRequest;
+  nsIRequest* mRequest;
 };
 
 // Used to suspend data events from mRequest within a function scope.  This is
@@ -58,19 +60,20 @@ nsBaseChannel::nsBaseChannel()
       mAllowThreadRetargeting(true),
       mWaitingOnAsyncRedirect(false),
       mOpenRedirectChannel(false),
+      mRedirectFlags{0},
       mStatus(NS_OK),
       mContentDispositionHint(UINT32_MAX),
       mContentLength(-1),
-      mWasOpened(false) {
+      mWasOpened(false),
+      mCanceled(false) {
   mContentType.AssignLiteral(UNKNOWN_CONTENT_TYPE);
 }
 
 nsBaseChannel::~nsBaseChannel() {
-  NS_ReleaseOnMainThreadSystemGroup("nsBaseChannel::mLoadInfo",
-                                    mLoadInfo.forget());
+  NS_ReleaseOnMainThread("nsBaseChannel::mLoadInfo", mLoadInfo.forget());
 }
 
-nsresult nsBaseChannel::Redirect(nsIChannel *newChannel, uint32_t redirectFlags,
+nsresult nsBaseChannel::Redirect(nsIChannel* newChannel, uint32_t redirectFlags,
                                  bool openNewChannel) {
   SUSPEND_PUMP_FOR_SCOPE();
 
@@ -82,55 +85,48 @@ nsresult nsBaseChannel::Redirect(nsIChannel *newChannel, uint32_t redirectFlags,
 
   // make a copy of the loadinfo, append to the redirectchain
   // and set it on the new channel
-  if (mLoadInfo) {
-    nsSecurityFlags secFlags = mLoadInfo->GetSecurityFlags() &
-                               ~nsILoadInfo::SEC_FORCE_INHERIT_PRINCIPAL;
-    nsCOMPtr<nsILoadInfo> newLoadInfo =
-        static_cast<mozilla::LoadInfo *>(mLoadInfo.get())
-            ->CloneWithNewSecFlags(secFlags);
+  nsSecurityFlags secFlags =
+      mLoadInfo->GetSecurityFlags() & ~nsILoadInfo::SEC_FORCE_INHERIT_PRINCIPAL;
+  nsCOMPtr<nsILoadInfo> newLoadInfo =
+      static_cast<net::LoadInfo*>(mLoadInfo.get())
+          ->CloneWithNewSecFlags(secFlags);
 
-    nsCOMPtr<nsIPrincipal> uriPrincipal;
-    nsIScriptSecurityManager *sm = nsContentUtils::GetSecurityManager();
-    sm->GetChannelURIPrincipal(this, getter_AddRefs(uriPrincipal));
-    bool isInternalRedirect =
-        (redirectFlags & (nsIChannelEventSink::REDIRECT_INTERNAL |
-                          nsIChannelEventSink::REDIRECT_STS_UPGRADE));
+  nsCOMPtr<nsIPrincipal> uriPrincipal;
+  nsIScriptSecurityManager* sm = nsContentUtils::GetSecurityManager();
+  sm->GetChannelURIPrincipal(this, getter_AddRefs(uriPrincipal));
+  bool isInternalRedirect =
+      (redirectFlags & (nsIChannelEventSink::REDIRECT_INTERNAL |
+                        nsIChannelEventSink::REDIRECT_STS_UPGRADE));
 
-    // nsBaseChannel hst no thing to do with HttpBaseChannel, we would not care
-    // about referrer and remote address in this case
-    nsCOMPtr<nsIRedirectHistoryEntry> entry =
-        new nsRedirectHistoryEntry(uriPrincipal, nullptr, EmptyCString());
+  // nsBaseChannel hst no thing to do with HttpBaseChannel, we would not care
+  // about referrer and remote address in this case
+  nsCOMPtr<nsIRedirectHistoryEntry> entry =
+      new net::nsRedirectHistoryEntry(uriPrincipal, nullptr, EmptyCString());
 
-    newLoadInfo->AppendRedirectHistoryEntry(entry, isInternalRedirect);
+  newLoadInfo->AppendRedirectHistoryEntry(entry, isInternalRedirect);
 
-    // Ensure the channel's loadInfo's result principal URI so that it's
-    // either non-null or updated to the redirect target URI.
-    // We must do this because in case the loadInfo's result principal URI
-    // is null, it would be taken from OriginalURI of the channel.  But we
-    // overwrite it with the whole redirect chain first URI before opening
-    // the target channel, hence the information would be lost.
-    // If the protocol handler that created the channel wants to use
-    // the originalURI of the channel as the principal URI, it has left
-    // the result principal URI on the load info null.
-    nsCOMPtr<nsIURI> resultPrincipalURI;
+  // Ensure the channel's loadInfo's result principal URI so that it's
+  // either non-null or updated to the redirect target URI.
+  // We must do this because in case the loadInfo's result principal URI
+  // is null, it would be taken from OriginalURI of the channel.  But we
+  // overwrite it with the whole redirect chain first URI before opening
+  // the target channel, hence the information would be lost.
+  // If the protocol handler that created the channel wants to use
+  // the originalURI of the channel as the principal URI, it has left
+  // the result principal URI on the load info null.
+  nsCOMPtr<nsIURI> resultPrincipalURI;
 
-    nsCOMPtr<nsILoadInfo> existingLoadInfo = newChannel->GetLoadInfo();
-    if (existingLoadInfo) {
-      existingLoadInfo->GetResultPrincipalURI(
-          getter_AddRefs(resultPrincipalURI));
-    }
-    if (!resultPrincipalURI) {
-      newChannel->GetOriginalURI(getter_AddRefs(resultPrincipalURI));
-    }
-
-    newLoadInfo->SetResultPrincipalURI(resultPrincipalURI);
-
-    newChannel->SetLoadInfo(newLoadInfo);
-  } else {
-    // the newChannel was created with a dummy loadInfo, we should clear
-    // it in case the original channel does not have a loadInfo
-    newChannel->SetLoadInfo(nullptr);
+  nsCOMPtr<nsILoadInfo> existingLoadInfo = newChannel->LoadInfo();
+  if (existingLoadInfo) {
+    existingLoadInfo->GetResultPrincipalURI(getter_AddRefs(resultPrincipalURI));
   }
+  if (!resultPrincipalURI) {
+    newChannel->GetOriginalURI(getter_AddRefs(resultPrincipalURI));
+  }
+
+  newLoadInfo->SetResultPrincipalURI(resultPrincipalURI);
+
+  newChannel->SetLoadInfo(newLoadInfo);
 
   // Preserve the privacy bit if it has been overridden
   if (mPrivateBrowsingOverriden) {
@@ -141,17 +137,13 @@ nsresult nsBaseChannel::Redirect(nsIChannel *newChannel, uint32_t redirectFlags,
     }
   }
 
-  nsCOMPtr<nsIWritablePropertyBag> bag = ::do_QueryInterface(newChannel);
-  if (bag) {
-    for (auto iter = mPropertyHash.Iter(); !iter.Done(); iter.Next()) {
-      bag->SetProperty(iter.Key(), iter.UserData());
-    }
+  if (nsCOMPtr<nsIWritablePropertyBag> bag = ::do_QueryInterface(newChannel)) {
+    nsHashPropertyBag::CopyFrom(bag, static_cast<nsIPropertyBag2*>(this));
   }
 
   // Notify consumer, giving chance to cancel redirect.
 
-  RefPtr<nsAsyncRedirectVerifyHelper> redirectCallbackHelper =
-      new nsAsyncRedirectVerifyHelper();
+  auto redirectCallbackHelper = MakeRefPtr<net::nsAsyncRedirectVerifyHelper>();
 
   bool checkRedirectSynchronously = !openNewChannel;
   nsCOMPtr<nsIEventTarget> target = GetNeckoTarget();
@@ -178,12 +170,7 @@ nsresult nsBaseChannel::ContinueRedirect() {
 
   if (mOpenRedirectChannel) {
     nsresult rv = NS_OK;
-    if (mLoadInfo && mLoadInfo->GetEnforceSecurity()) {
-      MOZ_ASSERT(!mListenerContext, "mListenerContext should be null!");
-      rv = mRedirectChannel->AsyncOpen2(mListener);
-    } else {
-      rv = mRedirectChannel->AsyncOpen(mListener, mListenerContext);
-    }
+    rv = mRedirectChannel->AsyncOpen(mListener);
     NS_ENSURE_SUCCESS(rv, rv);
   }
 
@@ -201,10 +188,10 @@ bool nsBaseChannel::HasContentTypeHint() const {
   return !mContentType.EqualsLiteral(UNKNOWN_CONTENT_TYPE);
 }
 
-nsresult nsBaseChannel::PushStreamConverter(const char *fromType,
-                                            const char *toType,
+nsresult nsBaseChannel::PushStreamConverter(const char* fromType,
+                                            const char* toType,
                                             bool invalidatesContentLength,
-                                            nsIStreamListener **result) {
+                                            nsIStreamListener** result) {
   NS_ASSERTION(mListener, "no listener");
 
   nsresult rv;
@@ -213,7 +200,7 @@ nsresult nsBaseChannel::PushStreamConverter(const char *fromType,
   if (NS_FAILED(rv)) return rv;
 
   nsCOMPtr<nsIStreamListener> converter;
-  rv = scs->AsyncConvertData(fromType, toType, mListener, mListenerContext,
+  rv = scs->AsyncConvertData(fromType, toType, mListener, nullptr,
                              getter_AddRefs(converter));
   if (NS_SUCCEEDED(rv)) {
     mListener = converter;
@@ -261,16 +248,49 @@ nsresult nsBaseChannel::BeginPumpingData() {
   nsCOMPtr<nsIEventTarget> target = GetNeckoTarget();
   rv = nsInputStreamPump::Create(getter_AddRefs(mPump), stream, 0, 0, true,
                                  target);
-  if (NS_SUCCEEDED(rv)) {
-    mPumpingData = true;
-    mRequest = mPump;
-    rv = mPump->AsyncRead(this, nullptr);
+  if (NS_FAILED(rv)) {
+    return rv;
   }
 
-  return rv;
+  mPumpingData = true;
+  mRequest = mPump;
+  rv = mPump->AsyncRead(this, nullptr);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  RefPtr<BlockingPromise> promise;
+  rv = ListenerBlockingPromise(getter_AddRefs(promise));
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+
+  if (promise) {
+    mPump->Suspend();
+
+    RefPtr<nsBaseChannel> self(this);
+    nsCOMPtr<nsISerialEventTarget> serialTarget(do_QueryInterface(target));
+    MOZ_ASSERT(serialTarget);
+
+    promise->Then(
+        serialTarget, __func__,
+        [self, this](nsresult rv) {
+          MOZ_ASSERT(mPump);
+          MOZ_ASSERT(NS_SUCCEEDED(rv));
+          mPump->Resume();
+        },
+        [self, this](nsresult rv) {
+          MOZ_ASSERT(mPump);
+          MOZ_ASSERT(NS_FAILED(rv));
+          Cancel(rv);
+          mPump->Resume();
+        });
+  }
+
+  return NS_OK;
 }
 
-void nsBaseChannel::HandleAsyncRedirect(nsIChannel *newChannel) {
+void nsBaseChannel::HandleAsyncRedirect(nsIChannel* newChannel) {
   NS_ASSERTION(!mPumpingData, "Shouldn't have gotten here");
 
   nsresult rv = mStatus;
@@ -292,8 +312,8 @@ void nsBaseChannel::ContinueHandleAsyncRedirect(nsresult result) {
 
   if (NS_FAILED(result) && mListener) {
     // Notify our consumer ourselves
-    mListener->OnStartRequest(this, mListenerContext);
-    mListener->OnStopRequest(this, mListenerContext, mStatus);
+    mListener->OnStartRequest(this);
+    mListener->OnStopRequest(this, mStatus);
     ChannelDone();
   }
 
@@ -311,8 +331,8 @@ void nsBaseChannel::ClassifyURI() {
     return;
   }
 
-  if (mLoadFlags & LOAD_CLASSIFY_URI) {
-    RefPtr<nsChannelClassifier> classifier = new nsChannelClassifier(this);
+  if (NS_ShouldClassifyChannel(this)) {
+    auto classifier = MakeRefPtr<net::nsChannelClassifier>(this);
     if (classifier) {
       classifier->Start();
     } else {
@@ -324,19 +344,27 @@ void nsBaseChannel::ClassifyURI() {
 //-----------------------------------------------------------------------------
 // nsBaseChannel::nsISupports
 
-NS_IMPL_ISUPPORTS_INHERITED(nsBaseChannel, nsHashPropertyBag, nsIRequest,
-                            nsIChannel, nsIThreadRetargetableRequest,
-                            nsIInterfaceRequestor, nsITransportEventSink,
-                            nsIRequestObserver, nsIStreamListener,
-                            nsIThreadRetargetableStreamListener,
-                            nsIAsyncVerifyRedirectCallback,
-                            nsIPrivateBrowsingChannel)
+NS_IMPL_ADDREF(nsBaseChannel)
+NS_IMPL_RELEASE(nsBaseChannel)
+
+NS_INTERFACE_MAP_BEGIN(nsBaseChannel)
+  NS_INTERFACE_MAP_ENTRY(nsIRequest)
+  NS_INTERFACE_MAP_ENTRY(nsIChannel)
+  NS_INTERFACE_MAP_ENTRY(nsIThreadRetargetableRequest)
+  NS_INTERFACE_MAP_ENTRY(nsIInterfaceRequestor)
+  NS_INTERFACE_MAP_ENTRY(nsITransportEventSink)
+  NS_INTERFACE_MAP_ENTRY(nsIRequestObserver)
+  NS_INTERFACE_MAP_ENTRY(nsIStreamListener)
+  NS_INTERFACE_MAP_ENTRY(nsIThreadRetargetableStreamListener)
+  NS_INTERFACE_MAP_ENTRY(nsIAsyncVerifyRedirectCallback)
+  NS_INTERFACE_MAP_ENTRY(nsIPrivateBrowsingChannel)
+NS_INTERFACE_MAP_END_INHERITING(nsHashPropertyBag)
 
 //-----------------------------------------------------------------------------
 // nsBaseChannel::nsIRequest
 
 NS_IMETHODIMP
-nsBaseChannel::GetName(nsACString &result) {
+nsBaseChannel::GetName(nsACString& result) {
   if (!mURI) {
     result.Truncate();
     return NS_OK;
@@ -345,13 +373,13 @@ nsBaseChannel::GetName(nsACString &result) {
 }
 
 NS_IMETHODIMP
-nsBaseChannel::IsPending(bool *result) {
+nsBaseChannel::IsPending(bool* result) {
   *result = Pending();
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetStatus(nsresult *status) {
+nsBaseChannel::GetStatus(nsresult* status) {
   if (mRequest && NS_SUCCEEDED(mStatus)) {
     mRequest->GetStatus(status);
   } else {
@@ -363,11 +391,16 @@ nsBaseChannel::GetStatus(nsresult *status) {
 NS_IMETHODIMP
 nsBaseChannel::Cancel(nsresult status) {
   // Ignore redundant cancelation
-  if (NS_FAILED(mStatus)) return NS_OK;
+  if (mCanceled) {
+    return NS_OK;
+  }
 
+  mCanceled = true;
   mStatus = status;
 
-  if (mRequest) mRequest->Cancel(status);
+  if (mRequest) {
+    mRequest->Cancel(status);
+  }
 
   return NS_OK;
 }
@@ -387,7 +420,7 @@ nsBaseChannel::Resume() {
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetLoadFlags(nsLoadFlags *aLoadFlags) {
+nsBaseChannel::GetLoadFlags(nsLoadFlags* aLoadFlags) {
   *aLoadFlags = mLoadFlags;
   return NS_OK;
 }
@@ -399,13 +432,24 @@ nsBaseChannel::SetLoadFlags(nsLoadFlags aLoadFlags) {
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetLoadGroup(nsILoadGroup **aLoadGroup) {
-  NS_IF_ADDREF(*aLoadGroup = mLoadGroup);
+nsBaseChannel::GetTRRMode(nsIRequest::TRRMode* aTRRMode) {
+  return GetTRRModeImpl(aTRRMode);
+}
+
+NS_IMETHODIMP
+nsBaseChannel::SetTRRMode(nsIRequest::TRRMode aTRRMode) {
+  return SetTRRModeImpl(aTRRMode);
+}
+
+NS_IMETHODIMP
+nsBaseChannel::GetLoadGroup(nsILoadGroup** aLoadGroup) {
+  nsCOMPtr<nsILoadGroup> loadGroup(mLoadGroup);
+  loadGroup.forget(aLoadGroup);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::SetLoadGroup(nsILoadGroup *aLoadGroup) {
+nsBaseChannel::SetLoadGroup(nsILoadGroup* aLoadGroup) {
   if (!CanSetLoadGroup(aLoadGroup)) {
     return NS_ERROR_FAILURE;
   }
@@ -420,39 +464,42 @@ nsBaseChannel::SetLoadGroup(nsILoadGroup *aLoadGroup) {
 // nsBaseChannel::nsIChannel
 
 NS_IMETHODIMP
-nsBaseChannel::GetOriginalURI(nsIURI **aURI) {
-  *aURI = OriginalURI();
-  NS_ADDREF(*aURI);
+nsBaseChannel::GetOriginalURI(nsIURI** aURI) {
+  RefPtr<nsIURI> uri = OriginalURI();
+  uri.forget(aURI);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::SetOriginalURI(nsIURI *aURI) {
+nsBaseChannel::SetOriginalURI(nsIURI* aURI) {
   NS_ENSURE_ARG_POINTER(aURI);
   mOriginalURI = aURI;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetURI(nsIURI **aURI) {
-  NS_IF_ADDREF(*aURI = mURI);
+nsBaseChannel::GetURI(nsIURI** aURI) {
+  nsCOMPtr<nsIURI> uri(mURI);
+  uri.forget(aURI);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetOwner(nsISupports **aOwner) {
-  NS_IF_ADDREF(*aOwner = mOwner);
+nsBaseChannel::GetOwner(nsISupports** aOwner) {
+  nsCOMPtr<nsISupports> owner(mOwner);
+  owner.forget(aOwner);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::SetOwner(nsISupports *aOwner) {
+nsBaseChannel::SetOwner(nsISupports* aOwner) {
   mOwner = aOwner;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::SetLoadInfo(nsILoadInfo *aLoadInfo) {
+nsBaseChannel::SetLoadInfo(nsILoadInfo* aLoadInfo) {
+  MOZ_RELEASE_ASSERT(aLoadInfo, "loadinfo can't be null");
   mLoadInfo = aLoadInfo;
 
   // Need to update |mNeckoTarget| when load info has changed.
@@ -461,24 +508,26 @@ nsBaseChannel::SetLoadInfo(nsILoadInfo *aLoadInfo) {
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetLoadInfo(nsILoadInfo **aLoadInfo) {
-  NS_IF_ADDREF(*aLoadInfo = mLoadInfo);
+nsBaseChannel::GetLoadInfo(nsILoadInfo** aLoadInfo) {
+  nsCOMPtr<nsILoadInfo> loadInfo(mLoadInfo);
+  loadInfo.forget(aLoadInfo);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetIsDocument(bool *aIsDocument) {
+nsBaseChannel::GetIsDocument(bool* aIsDocument) {
   return NS_GetIsDocumentChannel(this, aIsDocument);
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetNotificationCallbacks(nsIInterfaceRequestor **aCallbacks) {
-  NS_IF_ADDREF(*aCallbacks = mCallbacks);
+nsBaseChannel::GetNotificationCallbacks(nsIInterfaceRequestor** aCallbacks) {
+  nsCOMPtr<nsIInterfaceRequestor> callbacks(mCallbacks);
+  callbacks.forget(aCallbacks);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::SetNotificationCallbacks(nsIInterfaceRequestor *aCallbacks) {
+nsBaseChannel::SetNotificationCallbacks(nsIInterfaceRequestor* aCallbacks) {
   if (!CanSetCallbacks(aCallbacks)) {
     return NS_ERROR_FAILURE;
   }
@@ -490,19 +539,20 @@ nsBaseChannel::SetNotificationCallbacks(nsIInterfaceRequestor *aCallbacks) {
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetSecurityInfo(nsISupports **aSecurityInfo) {
-  NS_IF_ADDREF(*aSecurityInfo = mSecurityInfo);
+nsBaseChannel::GetSecurityInfo(nsISupports** aSecurityInfo) {
+  nsCOMPtr<nsISupports> securityInfo(mSecurityInfo);
+  securityInfo.forget(aSecurityInfo);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetContentType(nsACString &aContentType) {
+nsBaseChannel::GetContentType(nsACString& aContentType) {
   aContentType = mContentType;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::SetContentType(const nsACString &aContentType) {
+nsBaseChannel::SetContentType(const nsACString& aContentType) {
   // mContentCharset is unchanged if not parsed
   bool dummy;
   net_ParseContentType(aContentType, mContentType, mContentCharset, &dummy);
@@ -510,19 +560,19 @@ nsBaseChannel::SetContentType(const nsACString &aContentType) {
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetContentCharset(nsACString &aContentCharset) {
+nsBaseChannel::GetContentCharset(nsACString& aContentCharset) {
   aContentCharset = mContentCharset;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::SetContentCharset(const nsACString &aContentCharset) {
+nsBaseChannel::SetContentCharset(const nsACString& aContentCharset) {
   mContentCharset = aContentCharset;
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetContentDisposition(uint32_t *aContentDisposition) {
+nsBaseChannel::GetContentDisposition(uint32_t* aContentDisposition) {
   // preserve old behavior, fail unless explicitly set.
   if (mContentDispositionHint == UINT32_MAX) {
     return NS_ERROR_NOT_AVAILABLE;
@@ -540,7 +590,7 @@ nsBaseChannel::SetContentDisposition(uint32_t aContentDisposition) {
 
 NS_IMETHODIMP
 nsBaseChannel::GetContentDispositionFilename(
-    nsAString &aContentDispositionFilename) {
+    nsAString& aContentDispositionFilename) {
   if (!mContentDispositionFilename) {
     return NS_ERROR_NOT_AVAILABLE;
   }
@@ -551,19 +601,26 @@ nsBaseChannel::GetContentDispositionFilename(
 
 NS_IMETHODIMP
 nsBaseChannel::SetContentDispositionFilename(
-    const nsAString &aContentDispositionFilename) {
-  mContentDispositionFilename = new nsString(aContentDispositionFilename);
+    const nsAString& aContentDispositionFilename) {
+  mContentDispositionFilename =
+      MakeUnique<nsString>(aContentDispositionFilename);
+
+  // For safety reasons ensure the filename doesn't contain null characters and
+  // replace them with underscores. We may later pass the extension to system
+  // MIME APIs that expect null terminated strings.
+  mContentDispositionFilename->ReplaceChar(char16_t(0), '_');
+
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsBaseChannel::GetContentDispositionHeader(
-    nsACString &aContentDispositionHeader) {
+    nsACString& aContentDispositionHeader) {
   return NS_ERROR_NOT_AVAILABLE;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetContentLength(int64_t *aContentLength) {
+nsBaseChannel::GetContentLength(int64_t* aContentLength) {
   *aContentLength = mContentLength;
   return NS_OK;
 }
@@ -575,20 +632,25 @@ nsBaseChannel::SetContentLength(int64_t aContentLength) {
 }
 
 NS_IMETHODIMP
-nsBaseChannel::Open(nsIInputStream **result) {
+nsBaseChannel::Open(nsIInputStream** aStream) {
+  nsCOMPtr<nsIStreamListener> listener;
+  nsresult rv =
+      nsContentSecurityManager::doContentSecurityCheck(this, listener);
+  NS_ENSURE_SUCCESS(rv, rv);
+
   NS_ENSURE_TRUE(mURI, NS_ERROR_NOT_INITIALIZED);
   NS_ENSURE_TRUE(!mPumpingData, NS_ERROR_IN_PROGRESS);
   NS_ENSURE_TRUE(!mWasOpened, NS_ERROR_IN_PROGRESS);
 
   nsCOMPtr<nsIChannel> chan;
-  nsresult rv = OpenContentStream(false, result, getter_AddRefs(chan));
-  NS_ASSERTION(!chan || !*result, "Got both a channel and a stream?");
+  rv = OpenContentStream(false, aStream, getter_AddRefs(chan));
+  NS_ASSERTION(!chan || !*aStream, "Got both a channel and a stream?");
   if (NS_SUCCEEDED(rv) && chan) {
     rv = Redirect(chan, nsIChannelEventSink::REDIRECT_INTERNAL, false);
     if (NS_FAILED(rv)) return rv;
-    rv = chan->Open(result);
+    rv = chan->Open(aStream);
   } else if (rv == NS_ERROR_NOT_IMPLEMENTED)
-    return NS_ImplementChannelOpen(this, result);
+    return NS_ImplementChannelOpen(this, aStream);
 
   if (NS_SUCCEEDED(rv)) {
     mWasOpened = true;
@@ -599,23 +661,24 @@ nsBaseChannel::Open(nsIInputStream **result) {
 }
 
 NS_IMETHODIMP
-nsBaseChannel::Open2(nsIInputStream **aStream) {
-  nsCOMPtr<nsIStreamListener> listener;
+nsBaseChannel::AsyncOpen(nsIStreamListener* aListener) {
+  nsCOMPtr<nsIStreamListener> listener = aListener;
+
   nsresult rv =
       nsContentSecurityManager::doContentSecurityCheck(this, listener);
-  NS_ENSURE_SUCCESS(rv, rv);
-  return Open(aStream);
-}
+  if (NS_FAILED(rv)) {
+    mCallbacks = nullptr;
+    return rv;
+  }
 
-NS_IMETHODIMP
-nsBaseChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctxt) {
   MOZ_ASSERT(
-      !mLoadInfo || mLoadInfo->GetSecurityMode() == 0 ||
+      mLoadInfo->GetSecurityMode() == 0 ||
           mLoadInfo->GetInitialSecurityCheckDone() ||
           (mLoadInfo->GetSecurityMode() ==
                nsILoadInfo::SEC_ALLOW_CROSS_ORIGIN_DATA_IS_NULL &&
-           nsContentUtils::IsSystemPrincipal(mLoadInfo->LoadingPrincipal())),
-      "security flags in loadInfo but asyncOpen2() not called");
+           mLoadInfo->GetLoadingPrincipal() &&
+           mLoadInfo->GetLoadingPrincipal()->IsSystemPrincipal()),
+      "security flags in loadInfo but doContentSecurityCheck() not called");
 
   NS_ENSURE_TRUE(mURI, NS_ERROR_NOT_INITIALIZED);
   NS_ENSURE_TRUE(!mPumpingData, NS_ERROR_IN_PROGRESS);
@@ -632,7 +695,7 @@ nsBaseChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctxt) {
   }
 
   // Ensure that this is an allowed port before proceeding.
-  nsresult rv = NS_CheckPortSafety(mURI);
+  rv = NS_CheckPortSafety(mURI);
   if (NS_FAILED(rv)) {
     mCallbacks = nullptr;
     return rv;
@@ -644,7 +707,6 @@ nsBaseChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctxt) {
   // this typically introduces a reference cycle between this and the listener,
   // we need to be sure to break the reference if this method does not succeed.
   mListener = listener;
-  mListenerContext = ctxt;
 
   // This method assigns mPump as a side-effect.  We need to clear mPump if
   // this method fails.
@@ -671,23 +733,11 @@ nsBaseChannel::AsyncOpen(nsIStreamListener *listener, nsISupports *ctxt) {
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsBaseChannel::AsyncOpen2(nsIStreamListener *aListener) {
-  nsCOMPtr<nsIStreamListener> listener = aListener;
-  nsresult rv =
-      nsContentSecurityManager::doContentSecurityCheck(this, listener);
-  if (NS_FAILED(rv)) {
-    mCallbacks = nullptr;
-    return rv;
-  }
-  return AsyncOpen(listener, nullptr);
-}
-
 //-----------------------------------------------------------------------------
 // nsBaseChannel::nsITransportEventSink
 
 NS_IMETHODIMP
-nsBaseChannel::OnTransportStatus(nsITransport *transport, nsresult status,
+nsBaseChannel::OnTransportStatus(nsITransport* transport, nsresult status,
                                  int64_t progress, int64_t progressMax) {
   // In some cases, we may wish to suppress transport-layer status events.
 
@@ -712,12 +762,12 @@ nsBaseChannel::OnTransportStatus(nsITransport *transport, nsresult status,
   if (!HasLoadFlag(LOAD_BACKGROUND)) {
     nsAutoString statusArg;
     if (GetStatusArg(status, statusArg)) {
-      mProgressSink->OnStatus(this, mListenerContext, status, statusArg.get());
+      mProgressSink->OnStatus(this, status, statusArg.get());
     }
   }
 
   if (progress) {
-    mProgressSink->OnProgress(this, mListenerContext, progress, progressMax);
+    mProgressSink->OnProgress(this, progress, progressMax);
   }
 
   return NS_OK;
@@ -727,7 +777,7 @@ nsBaseChannel::OnTransportStatus(nsITransport *transport, nsresult status,
 // nsBaseChannel::nsIInterfaceRequestor
 
 NS_IMETHODIMP
-nsBaseChannel::GetInterface(const nsIID &iid, void **result) {
+nsBaseChannel::GetInterface(const nsIID& iid, void** result) {
   NS_QueryNotificationCallbacks(mCallbacks, mLoadGroup, iid, result);
   return *result ? NS_OK : NS_ERROR_NO_INTERFACE;
 }
@@ -735,9 +785,9 @@ nsBaseChannel::GetInterface(const nsIID &iid, void **result) {
 //-----------------------------------------------------------------------------
 // nsBaseChannel::nsIRequestObserver
 
-static void CallTypeSniffers(void *aClosure, const uint8_t *aData,
+static void CallTypeSniffers(void* aClosure, const uint8_t* aData,
                              uint32_t aCount) {
-  nsIChannel *chan = static_cast<nsIChannel *>(aClosure);
+  nsIChannel* chan = static_cast<nsIChannel*>(aClosure);
 
   nsAutoCString newType;
   NS_SniffContent(NS_CONTENT_SNIFFER_CATEGORY, chan, aData, aCount, newType);
@@ -746,13 +796,11 @@ static void CallTypeSniffers(void *aClosure, const uint8_t *aData,
   }
 }
 
-static void CallUnknownTypeSniffer(void *aClosure, const uint8_t *aData,
+static void CallUnknownTypeSniffer(void* aClosure, const uint8_t* aData,
                                    uint32_t aCount) {
-  nsIChannel *chan = static_cast<nsIChannel *>(aClosure);
+  nsIChannel* chan = static_cast<nsIChannel*>(aClosure);
 
-  nsCOMPtr<nsIContentSniffer> sniffer =
-      do_CreateInstance(NS_GENERIC_CONTENT_SNIFFER);
-  if (!sniffer) return;
+  RefPtr<nsUnknownDecoder> sniffer = new nsUnknownDecoder();
 
   nsAutoCString detected;
   nsresult rv = sniffer->GetMIMETypeFromContent(chan, aData, aCount, detected);
@@ -760,34 +808,35 @@ static void CallUnknownTypeSniffer(void *aClosure, const uint8_t *aData,
 }
 
 NS_IMETHODIMP
-nsBaseChannel::OnStartRequest(nsIRequest *request, nsISupports *ctxt) {
+nsBaseChannel::OnStartRequest(nsIRequest* request) {
   MOZ_ASSERT_IF(mRequest, request == mRequest);
 
-  if (mPump) {
+  nsAutoCString scheme;
+  mURI->GetScheme(scheme);
+
+  if (mPump && !scheme.EqualsLiteral("ftp")) {
     // If our content type is unknown, use the content type
     // sniffer. If the sniffer is not available for some reason, then we just
     // keep going as-is.
     if (NS_SUCCEEDED(mStatus) &&
         mContentType.EqualsLiteral(UNKNOWN_CONTENT_TYPE)) {
-      mPump->PeekStream(CallUnknownTypeSniffer,
-                        static_cast<nsIChannel *>(this));
+      mPump->PeekStream(CallUnknownTypeSniffer, static_cast<nsIChannel*>(this));
     }
 
     // Now, the general type sniffers. Skip this if we have none.
     if (mLoadFlags & LOAD_CALL_CONTENT_SNIFFERS)
-      mPump->PeekStream(CallTypeSniffers, static_cast<nsIChannel *>(this));
+      mPump->PeekStream(CallTypeSniffers, static_cast<nsIChannel*>(this));
   }
 
   SUSPEND_PUMP_FOR_SCOPE();
 
   if (mListener)  // null in case of redirect
-    return mListener->OnStartRequest(this, mListenerContext);
+    return mListener->OnStartRequest(this);
   return NS_OK;
 }
 
 NS_IMETHODIMP
-nsBaseChannel::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
-                             nsresult status) {
+nsBaseChannel::OnStopRequest(nsIRequest* request, nsresult status) {
   // If both mStatus and status are failure codes, we keep mStatus as-is since
   // that is consistent with our GetStatus and Cancel methods.
   if (NS_SUCCEEDED(mStatus)) mStatus = status;
@@ -798,7 +847,7 @@ nsBaseChannel::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
   mPumpingData = false;
 
   if (mListener)  // null in case of redirect
-    mListener->OnStopRequest(this, mListenerContext, mStatus);
+    mListener->OnStopRequest(this, mStatus);
   ChannelDone();
 
   // No need to suspend pump in this scope since we will not be receiving
@@ -817,27 +866,25 @@ nsBaseChannel::OnStopRequest(nsIRequest *request, nsISupports *ctxt,
 // nsBaseChannel::nsIStreamListener
 
 NS_IMETHODIMP
-nsBaseChannel::OnDataAvailable(nsIRequest *request, nsISupports *ctxt,
-                               nsIInputStream *stream, uint64_t offset,
-                               uint32_t count) {
+nsBaseChannel::OnDataAvailable(nsIRequest* request, nsIInputStream* stream,
+                               uint64_t offset, uint32_t count) {
   SUSPEND_PUMP_FOR_SCOPE();
 
-  nsresult rv =
-      mListener->OnDataAvailable(this, mListenerContext, stream, offset, count);
+  nsresult rv = mListener->OnDataAvailable(this, stream, offset, count);
   if (mSynthProgressEvents && NS_SUCCEEDED(rv)) {
     int64_t prog = offset + count;
     if (NS_IsMainThread()) {
       OnTransportStatus(nullptr, NS_NET_STATUS_READING, prog, mContentLength);
     } else {
-      class OnTransportStatusAsyncEvent : public mozilla::Runnable {
+      class OnTransportStatusAsyncEvent : public Runnable {
         RefPtr<nsBaseChannel> mChannel;
         int64_t mProgress;
         int64_t mContentLength;
 
        public:
-        OnTransportStatusAsyncEvent(nsBaseChannel *aChannel, int64_t aProgress,
+        OnTransportStatusAsyncEvent(nsBaseChannel* aChannel, int64_t aProgress,
                                     int64_t aContentLength)
-            : mozilla::Runnable("OnTransportStatusAsyncEvent"),
+            : Runnable("OnTransportStatusAsyncEvent"),
               mChannel(aChannel),
               mProgress(aProgress),
               mContentLength(aContentLength) {}
@@ -872,7 +919,7 @@ nsBaseChannel::OnRedirectVerifyCallback(nsresult result) {
 }
 
 NS_IMETHODIMP
-nsBaseChannel::RetargetDeliveryTo(nsIEventTarget *aEventTarget) {
+nsBaseChannel::RetargetDeliveryTo(nsIEventTarget* aEventTarget) {
   MOZ_ASSERT(NS_IsMainThread());
 
   NS_ENSURE_TRUE(mRequest, NS_ERROR_NOT_INITIALIZED);
@@ -888,7 +935,7 @@ nsBaseChannel::RetargetDeliveryTo(nsIEventTarget *aEventTarget) {
 }
 
 NS_IMETHODIMP
-nsBaseChannel::GetDeliveryTarget(nsIEventTarget **aEventTarget) {
+nsBaseChannel::GetDeliveryTarget(nsIEventTarget** aEventTarget) {
   MOZ_ASSERT(NS_IsMainThread());
 
   NS_ENSURE_TRUE(mRequest, NS_ERROR_NOT_INITIALIZED);
@@ -917,6 +964,11 @@ nsBaseChannel::CheckListenerChain() {
   return listener->CheckListenerChain();
 }
 
+NS_IMETHODIMP nsBaseChannel::GetCanceled(bool* aCanceled) {
+  *aCanceled = mCanceled;
+  return NS_OK;
+}
+
 void nsBaseChannel::SetupNeckoTarget() {
   mNeckoTarget =
       nsContentUtils::GetEventTargetByLoadInfo(mLoadInfo, TaskCategory::Other);
