@@ -21,6 +21,7 @@
 #include "mozilla/StaticPtr.h"
 #include "mozilla/EndianUtils.h"
 #include "mozilla/TimeStamp.h"
+#include "mozilla/UniquePtr.h"
 
 class nsIFile;
 class nsIDirectoryEnumerator;
@@ -73,7 +74,6 @@ struct CacheIndexRecord {
   uint16_t mOnStartTime;
   uint16_t mOnStopTime;
   uint8_t mContentType;
-  uint16_t mBaseDomainAccessCount;
 
   /*
    *    1000 0000 0000 0000 0000 0000 0000 0000 : initialized
@@ -94,7 +94,6 @@ struct CacheIndexRecord {
         mOnStartTime(kIndexTimeNotAvailable),
         mOnStopTime(kIndexTimeNotAvailable),
         mContentType(nsICacheEntry::CONTENT_TYPE_UNKNOWN),
-        mBaseDomainAccessCount(0),
         mFlags(0) {}
 };
 #pragma pack(pop)
@@ -105,11 +104,22 @@ static_assert(sizeof(CacheIndexRecord::mHash) +
                       sizeof(CacheIndexRecord::mOnStartTime) +
                       sizeof(CacheIndexRecord::mOnStopTime) +
                       sizeof(CacheIndexRecord::mContentType) +
-                      sizeof(CacheIndexRecord::mBaseDomainAccessCount) +
                       sizeof(CacheIndexRecord::mFlags) ==
                   sizeof(CacheIndexRecord),
               "Unexpected sizeof(CacheIndexRecord)!");
 
+class CacheIndexRecordWrapper final {
+ public:
+  NS_INLINE_DECL_THREADSAFE_REFCOUNTING(CacheIndexRecordWrapper)
+
+  CacheIndexRecordWrapper() : mRec(MakeUnique<CacheIndexRecord>()) {}
+  CacheIndexRecord* Get() { return mRec.get(); }
+
+ private:
+  ~CacheIndexRecordWrapper() = default;
+  UniquePtr<CacheIndexRecord> mRec;
+};
+
 class CacheIndexEntry : public PLDHashEntryHdr {
  public:
   typedef const SHA1Sum::Hash& KeyType;
@@ -117,10 +127,10 @@ class CacheIndexEntry : public PLDHashEntryHdr {
 
   explicit CacheIndexEntry(KeyTypePointer aKey) {
     MOZ_COUNT_CTOR(CacheIndexEntry);
-    mRec = new CacheIndexRecord();
+    mRec = new CacheIndexRecordWrapper();
     LOG(("CacheIndexEntry::CacheIndexEntry() - Created record [rec=%p]",
-         mRec.get()));
-    memcpy(&mRec->mHash, aKey, sizeof(SHA1Sum::Hash));
+         mRec->Get()));
+    memcpy(&mRec->Get()->mHash, aKey, sizeof(SHA1Sum::Hash));
   }
   CacheIndexEntry(const CacheIndexEntry& aOther) {
     MOZ_ASSERT_UNREACHABLE("CacheIndexEntry copy constructor is forbidden!");
@@ -128,12 +138,12 @@ class CacheIndexEntry : public PLDHashEntryHdr {
   ~CacheIndexEntry() {
     MOZ_COUNT_DTOR(CacheIndexEntry);
     LOG(("CacheIndexEntry::~CacheIndexEntry() - Deleting record [rec=%p]",
-         mRec.get()));
+         mRec->Get()));
   }
 
   // KeyEquals(): does this entry match this key?
   bool KeyEquals(KeyTypePointer aKey) const {
-    return memcmp(&mRec->mHash, aKey, sizeof(SHA1Sum::Hash)) == 0;
+    return memcmp(&mRec->Get()->mHash, aKey, sizeof(SHA1Sum::Hash)) == 0;
   }
 
   // KeyToPointer(): Convert KeyType to KeyTypePointer
@@ -149,97 +159,102 @@ class CacheIndexEntry : public PLDHashEntryHdr {
   enum { ALLOW_MEMMOVE = true };
 
   bool operator==(const CacheIndexEntry& aOther) const {
-    return KeyEquals(&aOther.mRec->mHash);
+    return KeyEquals(&aOther.mRec->Get()->mHash);
   }
 
   CacheIndexEntry& operator=(const CacheIndexEntry& aOther) {
-    MOZ_ASSERT(
-        memcmp(&mRec->mHash, &aOther.mRec->mHash, sizeof(SHA1Sum::Hash)) == 0);
-    mRec->mFrecency = aOther.mRec->mFrecency;
-    mRec->mOriginAttrsHash = aOther.mRec->mOriginAttrsHash;
-    mRec->mOnStartTime = aOther.mRec->mOnStartTime;
-    mRec->mOnStopTime = aOther.mRec->mOnStopTime;
-    mRec->mContentType = aOther.mRec->mContentType;
-    mRec->mBaseDomainAccessCount = aOther.mRec->mBaseDomainAccessCount;
-    mRec->mFlags = aOther.mRec->mFlags;
+    MOZ_ASSERT(memcmp(&mRec->Get()->mHash, &aOther.mRec->Get()->mHash,
+                      sizeof(SHA1Sum::Hash)) == 0);
+    mRec->Get()->mFrecency = aOther.mRec->Get()->mFrecency;
+    mRec->Get()->mOriginAttrsHash = aOther.mRec->Get()->mOriginAttrsHash;
+    mRec->Get()->mOnStartTime = aOther.mRec->Get()->mOnStartTime;
+    mRec->Get()->mOnStopTime = aOther.mRec->Get()->mOnStopTime;
+    mRec->Get()->mContentType = aOther.mRec->Get()->mContentType;
+    mRec->Get()->mFlags = aOther.mRec->Get()->mFlags;
     return *this;
   }
 
   void InitNew() {
-    mRec->mFrecency = 0;
-    mRec->mOriginAttrsHash = 0;
-    mRec->mOnStartTime = kIndexTimeNotAvailable;
-    mRec->mOnStopTime = kIndexTimeNotAvailable;
-    mRec->mContentType = nsICacheEntry::CONTENT_TYPE_UNKNOWN;
-    mRec->mBaseDomainAccessCount = 0;
-    mRec->mFlags = 0;
+    mRec->Get()->mFrecency = 0;
+    mRec->Get()->mOriginAttrsHash = 0;
+    mRec->Get()->mOnStartTime = kIndexTimeNotAvailable;
+    mRec->Get()->mOnStopTime = kIndexTimeNotAvailable;
+    mRec->Get()->mContentType = nsICacheEntry::CONTENT_TYPE_UNKNOWN;
+    mRec->Get()->mFlags = 0;
   }
 
   void Init(OriginAttrsHash aOriginAttrsHash, bool aAnonymous, bool aPinned) {
-    MOZ_ASSERT(mRec->mFrecency == 0);
-    MOZ_ASSERT(mRec->mOriginAttrsHash == 0);
-    MOZ_ASSERT(mRec->mOnStartTime == kIndexTimeNotAvailable);
-    MOZ_ASSERT(mRec->mOnStopTime == kIndexTimeNotAvailable);
-    MOZ_ASSERT(mRec->mContentType == nsICacheEntry::CONTENT_TYPE_UNKNOWN);
-    MOZ_ASSERT(mRec->mBaseDomainAccessCount == 0);
+    MOZ_ASSERT(mRec->Get()->mFrecency == 0);
+    MOZ_ASSERT(mRec->Get()->mOriginAttrsHash == 0);
+    MOZ_ASSERT(mRec->Get()->mOnStartTime == kIndexTimeNotAvailable);
+    MOZ_ASSERT(mRec->Get()->mOnStopTime == kIndexTimeNotAvailable);
+    MOZ_ASSERT(mRec->Get()->mContentType ==
+               nsICacheEntry::CONTENT_TYPE_UNKNOWN);
     // When we init the entry it must be fresh and may be dirty
-    MOZ_ASSERT((mRec->mFlags & ~kDirtyMask) == kFreshMask);
+    MOZ_ASSERT((mRec->Get()->mFlags & ~kDirtyMask) == kFreshMask);
 
-    mRec->mOriginAttrsHash = aOriginAttrsHash;
-    mRec->mFlags |= kInitializedMask;
+    mRec->Get()->mOriginAttrsHash = aOriginAttrsHash;
+    mRec->Get()->mFlags |= kInitializedMask;
     if (aAnonymous) {
-      mRec->mFlags |= kAnonymousMask;
+      mRec->Get()->mFlags |= kAnonymousMask;
     }
     if (aPinned) {
-      mRec->mFlags |= kPinnedMask;
+      mRec->Get()->mFlags |= kPinnedMask;
     }
   }
 
-  const SHA1Sum::Hash* Hash() const { return &mRec->mHash; }
+  const SHA1Sum::Hash* Hash() const { return &mRec->Get()->mHash; }
 
-  bool IsInitialized() const { return !!(mRec->mFlags & kInitializedMask); }
+  bool IsInitialized() const {
+    return !!(mRec->Get()->mFlags & kInitializedMask);
+  }
 
   mozilla::net::OriginAttrsHash OriginAttrsHash() const {
-    return mRec->mOriginAttrsHash;
+    return mRec->Get()->mOriginAttrsHash;
   }
 
-  bool Anonymous() const { return !!(mRec->mFlags & kAnonymousMask); }
+  bool Anonymous() const { return !!(mRec->Get()->mFlags & kAnonymousMask); }
 
-  bool IsRemoved() const { return !!(mRec->mFlags & kRemovedMask); }
-  void MarkRemoved() { mRec->mFlags |= kRemovedMask; }
+  bool IsRemoved() const { return !!(mRec->Get()->mFlags & kRemovedMask); }
+  void MarkRemoved() { mRec->Get()->mFlags |= kRemovedMask; }
 
-  bool IsDirty() const { return !!(mRec->mFlags & kDirtyMask); }
-  void MarkDirty() { mRec->mFlags |= kDirtyMask; }
-  void ClearDirty() { mRec->mFlags &= ~kDirtyMask; }
+  bool IsDirty() const { return !!(mRec->Get()->mFlags & kDirtyMask); }
+  void MarkDirty() { mRec->Get()->mFlags |= kDirtyMask; }
+  void ClearDirty() { mRec->Get()->mFlags &= ~kDirtyMask; }
 
-  bool IsFresh() const { return !!(mRec->mFlags & kFreshMask); }
-  void MarkFresh() { mRec->mFlags |= kFreshMask; }
+  bool IsFresh() const { return !!(mRec->Get()->mFlags & kFreshMask); }
+  void MarkFresh() { mRec->Get()->mFlags |= kFreshMask; }
 
-  bool IsPinned() const { return !!(mRec->mFlags & kPinnedMask); }
+  bool IsPinned() const { return !!(mRec->Get()->mFlags & kPinnedMask); }
 
-  void SetFrecency(uint32_t aFrecency) { mRec->mFrecency = aFrecency; }
-  uint32_t GetFrecency() const { return mRec->mFrecency; }
+  void SetFrecency(uint32_t aFrecency) { mRec->Get()->mFrecency = aFrecency; }
+  uint32_t GetFrecency() const { return mRec->Get()->mFrecency; }
 
   void SetHasAltData(bool aHasAltData) {
-    aHasAltData ? mRec->mFlags |= kHasAltDataMask
-                : mRec->mFlags &= ~kHasAltDataMask;
+    aHasAltData ? mRec->Get()->mFlags |= kHasAltDataMask
+                : mRec->Get()->mFlags &= ~kHasAltDataMask;
   }
-  bool GetHasAltData() const { return !!(mRec->mFlags & kHasAltDataMask); }
-
-  void SetOnStartTime(uint16_t aTime) { mRec->mOnStartTime = aTime; }
-  uint16_t GetOnStartTime() const { return mRec->mOnStartTime; }
-
-  void SetOnStopTime(uint16_t aTime) { mRec->mOnStopTime = aTime; }
-  uint16_t GetOnStopTime() const { return mRec->mOnStopTime; }
-
-  void SetContentType(uint8_t aType) { mRec->mContentType = aType; }
-  uint8_t GetContentType() const { return mRec->mContentType; }
-
-  void SetBaseDomainAccessCount(uint16_t aCount) {
-    mRec->mBaseDomainAccessCount = aCount;
+  bool GetHasAltData() const {
+    return !!(mRec->Get()->mFlags & kHasAltDataMask);
   }
-  uint8_t GetBaseDomainAccessCount() const {
-    return mRec->mBaseDomainAccessCount;
+
+  void SetOnStartTime(uint16_t aTime) { mRec->Get()->mOnStartTime = aTime; }
+  uint16_t GetOnStartTime() const { return mRec->Get()->mOnStartTime; }
+
+  void SetOnStopTime(uint16_t aTime) { mRec->Get()->mOnStopTime = aTime; }
+  uint16_t GetOnStopTime() const { return mRec->Get()->mOnStopTime; }
+
+  void SetContentType(uint8_t aType) { mRec->Get()->mContentType = aType; }
+  uint8_t GetContentType() const { return GetContentType(mRec->Get()); }
+  static uint8_t GetContentType(CacheIndexRecord* aRec) {
+    if (aRec->mContentType >= nsICacheEntry::CONTENT_TYPE_LAST) {
+      LOG(
+          ("CacheIndexEntry::GetContentType() - Found invalid content type "
+           "[hash=%08x%08x%08x%08x%08x, contentType=%u]",
+           LOGSHA1(aRec->mHash), aRec->mContentType));
+      return nsICacheEntry::CONTENT_TYPE_UNKNOWN;
+    }
+    return aRec->mContentType;
   }
 
   // Sets filesize in kilobytes.
@@ -251,13 +266,13 @@ class CacheIndexEntry : public PLDHashEntryHdr {
            kFileSizeMask));
       aFileSize = kFileSizeMask;
     }
-    mRec->mFlags &= ~kFileSizeMask;
-    mRec->mFlags |= aFileSize;
+    mRec->Get()->mFlags &= ~kFileSizeMask;
+    mRec->Get()->mFlags |= aFileSize;
   }
   // Returns filesize in kilobytes.
-  uint32_t GetFileSize() const { return GetFileSize(mRec); }
-  static uint32_t GetFileSize(CacheIndexRecord* aRec) {
-    return aRec->mFlags & kFileSizeMask;
+  uint32_t GetFileSize() const { return GetFileSize(*(mRec->Get())); }
+  static uint32_t GetFileSize(const CacheIndexRecord& aRec) {
+    return aRec.mFlags & kFileSizeMask;
   }
   static uint32_t IsPinned(CacheIndexRecord* aRec) {
     return aRec->mFlags & kPinnedMask;
@@ -266,42 +281,39 @@ class CacheIndexEntry : public PLDHashEntryHdr {
 
   void WriteToBuf(void* aBuf) {
     uint8_t* ptr = static_cast<uint8_t*>(aBuf);
-    memcpy(ptr, mRec->mHash, sizeof(SHA1Sum::Hash));
+    memcpy(ptr, mRec->Get()->mHash, sizeof(SHA1Sum::Hash));
     ptr += sizeof(SHA1Sum::Hash);
-    NetworkEndian::writeUint32(ptr, mRec->mFrecency);
+    NetworkEndian::writeUint32(ptr, mRec->Get()->mFrecency);
     ptr += sizeof(uint32_t);
-    NetworkEndian::writeUint64(ptr, mRec->mOriginAttrsHash);
+    NetworkEndian::writeUint64(ptr, mRec->Get()->mOriginAttrsHash);
     ptr += sizeof(uint64_t);
-    NetworkEndian::writeUint16(ptr, mRec->mOnStartTime);
+    NetworkEndian::writeUint16(ptr, mRec->Get()->mOnStartTime);
     ptr += sizeof(uint16_t);
-    NetworkEndian::writeUint16(ptr, mRec->mOnStopTime);
+    NetworkEndian::writeUint16(ptr, mRec->Get()->mOnStopTime);
     ptr += sizeof(uint16_t);
-    *ptr = mRec->mContentType;
+    *ptr = mRec->Get()->mContentType;
     ptr += sizeof(uint8_t);
-    NetworkEndian::writeUint16(ptr, mRec->mBaseDomainAccessCount);
-    ptr += sizeof(uint16_t);
     // Dirty and fresh flags should never go to disk, since they make sense only
     // during current session.
-    NetworkEndian::writeUint32(ptr, mRec->mFlags & ~(kDirtyMask | kFreshMask));
+    NetworkEndian::writeUint32(
+        ptr, mRec->Get()->mFlags & ~(kDirtyMask | kFreshMask));
   }
 
   void ReadFromBuf(void* aBuf) {
     const uint8_t* ptr = static_cast<const uint8_t*>(aBuf);
-    MOZ_ASSERT(memcmp(&mRec->mHash, ptr, sizeof(SHA1Sum::Hash)) == 0);
+    MOZ_ASSERT(memcmp(&mRec->Get()->mHash, ptr, sizeof(SHA1Sum::Hash)) == 0);
     ptr += sizeof(SHA1Sum::Hash);
-    mRec->mFrecency = NetworkEndian::readUint32(ptr);
+    mRec->Get()->mFrecency = NetworkEndian::readUint32(ptr);
     ptr += sizeof(uint32_t);
-    mRec->mOriginAttrsHash = NetworkEndian::readUint64(ptr);
+    mRec->Get()->mOriginAttrsHash = NetworkEndian::readUint64(ptr);
     ptr += sizeof(uint64_t);
-    mRec->mOnStartTime = NetworkEndian::readUint16(ptr);
+    mRec->Get()->mOnStartTime = NetworkEndian::readUint16(ptr);
     ptr += sizeof(uint16_t);
-    mRec->mOnStopTime = NetworkEndian::readUint16(ptr);
+    mRec->Get()->mOnStopTime = NetworkEndian::readUint16(ptr);
     ptr += sizeof(uint16_t);
-    mRec->mContentType = *ptr;
+    mRec->Get()->mContentType = *ptr;
     ptr += sizeof(uint8_t);
-    mRec->mBaseDomainAccessCount = NetworkEndian::readUint16(ptr);
-    ptr += sizeof(uint16_t);
-    mRec->mFlags = NetworkEndian::readUint32(ptr);
+    mRec->Get()->mFlags = NetworkEndian::readUint32(ptr);
   }
 
   void Log() const {
@@ -309,22 +321,21 @@ class CacheIndexEntry : public PLDHashEntryHdr {
         ("CacheIndexEntry::Log() [this=%p, hash=%08x%08x%08x%08x%08x, fresh=%u,"
          " initialized=%u, removed=%u, dirty=%u, anonymous=%u, "
          "originAttrsHash=%" PRIx64 ", frecency=%u, hasAltData=%u, "
-         "onStartTime=%u, onStopTime=%u, contentType=%u, "
-         "baseDomainAccessCount=%u, size=%u]",
-         this, LOGSHA1(mRec->mHash), IsFresh(), IsInitialized(), IsRemoved(),
-         IsDirty(), Anonymous(), OriginAttrsHash(), GetFrecency(),
+         "onStartTime=%u, onStopTime=%u, contentType=%u, size=%u]",
+         this, LOGSHA1(mRec->Get()->mHash), IsFresh(), IsInitialized(),
+         IsRemoved(), IsDirty(), Anonymous(), OriginAttrsHash(), GetFrecency(),
          GetHasAltData(), GetOnStartTime(), GetOnStopTime(), GetContentType(),
-         GetBaseDomainAccessCount(), GetFileSize()));
+         GetFileSize()));
   }
 
-  static bool RecordMatchesLoadContextInfo(CacheIndexRecord* aRec,
+  static bool RecordMatchesLoadContextInfo(CacheIndexRecordWrapper* aRec,
                                            nsILoadContextInfo* aInfo) {
     MOZ_ASSERT(aInfo);
 
     if (!aInfo->IsPrivate() &&
         GetOriginAttrsHash(*aInfo->OriginAttributesPtr()) ==
-            aRec->mOriginAttrsHash &&
-        aInfo->IsAnonymous() == !!(aRec->mFlags & kAnonymousMask)) {
+            aRec->Get()->mOriginAttrsHash &&
+        aInfo->IsAnonymous() == !!(aRec->Get()->mFlags & kAnonymousMask)) {
       return true;
     }
 
@@ -333,7 +344,7 @@ class CacheIndexEntry : public PLDHashEntryHdr {
 
   // Memory reporting
   size_t SizeOfExcludingThis(mozilla::MallocSizeOf mallocSizeOf) const {
-    return mallocSizeOf(mRec.get());
+    return mallocSizeOf(mRec->Get());
   }
 
   size_t SizeOfIncludingThis(mozilla::MallocSizeOf mallocSizeOf) const {
@@ -371,7 +382,7 @@ class CacheIndexEntry : public PLDHashEntryHdr {
   // FileSize in kilobytes
   static const uint32_t kFileSizeMask = 0x00FFFFFF;
 
-  nsAutoPtr<CacheIndexRecord> mRec;
+  RefPtr<CacheIndexRecordWrapper> mRec;
 };
 
 class CacheIndexEntryUpdate : public CacheIndexEntry {
@@ -387,8 +398,8 @@ class CacheIndexEntryUpdate : public CacheIndexEntry {
   }
 
   CacheIndexEntryUpdate& operator=(const CacheIndexEntry& aOther) {
-    MOZ_ASSERT(
-        memcmp(&mRec->mHash, &aOther.mRec->mHash, sizeof(SHA1Sum::Hash)) == 0);
+    MOZ_ASSERT(memcmp(&mRec->Get()->mHash, &aOther.mRec->Get()->mHash,
+                      sizeof(SHA1Sum::Hash)) == 0);
     mUpdateFlags = 0;
     *(static_cast<CacheIndexEntry*>(this)) = aOther;
     return *this;
@@ -426,48 +437,41 @@ class CacheIndexEntryUpdate : public CacheIndexEntry {
     CacheIndexEntry::SetContentType(aType);
   }
 
-  void SetBaseDomainAccessCount(uint16_t aCount) {
-    mUpdateFlags |= kBaseDomainAccessCountUpdatedMask;
-    CacheIndexEntry::SetBaseDomainAccessCount(aCount);
-  }
-
   void SetFileSize(uint32_t aFileSize) {
     mUpdateFlags |= kFileSizeUpdatedMask;
     CacheIndexEntry::SetFileSize(aFileSize);
   }
 
   void ApplyUpdate(CacheIndexEntry* aDst) {
-    MOZ_ASSERT(
-        memcmp(&mRec->mHash, &aDst->mRec->mHash, sizeof(SHA1Sum::Hash)) == 0);
+    MOZ_ASSERT(memcmp(&mRec->Get()->mHash, &aDst->mRec->Get()->mHash,
+                      sizeof(SHA1Sum::Hash)) == 0);
     if (mUpdateFlags & kFrecencyUpdatedMask) {
-      aDst->mRec->mFrecency = mRec->mFrecency;
+      aDst->mRec->Get()->mFrecency = mRec->Get()->mFrecency;
     }
-    aDst->mRec->mOriginAttrsHash = mRec->mOriginAttrsHash;
+    aDst->mRec->Get()->mOriginAttrsHash = mRec->Get()->mOriginAttrsHash;
     if (mUpdateFlags & kOnStartTimeUpdatedMask) {
-      aDst->mRec->mOnStartTime = mRec->mOnStartTime;
+      aDst->mRec->Get()->mOnStartTime = mRec->Get()->mOnStartTime;
     }
     if (mUpdateFlags & kOnStopTimeUpdatedMask) {
-      aDst->mRec->mOnStopTime = mRec->mOnStopTime;
+      aDst->mRec->Get()->mOnStopTime = mRec->Get()->mOnStopTime;
     }
     if (mUpdateFlags & kContentTypeUpdatedMask) {
-      aDst->mRec->mContentType = mRec->mContentType;
-    }
-    if (mUpdateFlags & kBaseDomainAccessCountUpdatedMask) {
-      aDst->mRec->mBaseDomainAccessCount = mRec->mBaseDomainAccessCount;
+      aDst->mRec->Get()->mContentType = mRec->Get()->mContentType;
     }
     if (mUpdateFlags & kHasAltDataUpdatedMask &&
-        ((aDst->mRec->mFlags ^ mRec->mFlags) & kHasAltDataMask)) {
+        ((aDst->mRec->Get()->mFlags ^ mRec->Get()->mFlags) & kHasAltDataMask)) {
       // Toggle the bit if we need to.
-      aDst->mRec->mFlags ^= kHasAltDataMask;
+      aDst->mRec->Get()->mFlags ^= kHasAltDataMask;
     }
 
     if (mUpdateFlags & kFileSizeUpdatedMask) {
       // Copy all flags except |HasAltData|.
-      aDst->mRec->mFlags |= (mRec->mFlags & ~kHasAltDataMask);
+      aDst->mRec->Get()->mFlags |= (mRec->Get()->mFlags & ~kHasAltDataMask);
     } else {
       // Copy all flags except |HasAltData| and file size.
-      aDst->mRec->mFlags &= kFileSizeMask;
-      aDst->mRec->mFlags |= (mRec->mFlags & ~kHasAltDataMask & ~kFileSizeMask);
+      aDst->mRec->Get()->mFlags &= kFileSizeMask;
+      aDst->mRec->Get()->mFlags |=
+          (mRec->Get()->mFlags & ~kHasAltDataMask & ~kFileSizeMask);
     }
   }
 
@@ -478,7 +482,6 @@ class CacheIndexEntryUpdate : public CacheIndexEntry {
   static const uint32_t kHasAltDataUpdatedMask = 0x00000008;
   static const uint32_t kOnStartTimeUpdatedMask = 0x00000010;
   static const uint32_t kOnStopTimeUpdatedMask = 0x00000020;
-  static const uint32_t kBaseDomainAccessCountUpdatedMask = 0x00000040;
 
   uint32_t mUpdateFlags;
 };
@@ -499,9 +502,20 @@ class CacheIndexStats {
         mDisableLogging(false)
 #endif
   {
+    for (uint32_t i = 0; i < nsICacheEntry::CONTENT_TYPE_LAST; ++i) {
+      mCountByType[i] = 0;
+      mSizeByType[i] = 0;
+    }
   }
 
   bool operator==(const CacheIndexStats& aOther) const {
+    for (uint32_t i = 0; i < nsICacheEntry::CONTENT_TYPE_LAST; ++i) {
+      if (mCountByType[i] != aOther.mCountByType[i] ||
+          mSizeByType[i] != aOther.mSizeByType[i]) {
+        return false;
+      }
+    }
+
     return
 #ifdef DEBUG
         aOther.mStateLogged == mStateLogged &&
@@ -533,6 +547,10 @@ class CacheIndexStats {
     mFresh = 0;
     mEmpty = 0;
     mSize = 0;
+    for (uint32_t i = 0; i < nsICacheEntry::CONTENT_TYPE_LAST; ++i) {
+      mCountByType[i] = 0;
+      mSizeByType[i] = 0;
+    }
   }
 
 #ifdef DEBUG
@@ -544,6 +562,12 @@ class CacheIndexStats {
     return mCount;
   }
 
+  uint32_t CountByType(uint8_t aContentType) {
+    MOZ_ASSERT(!mStateLogged, "CacheIndexStats::CountByType() - state logged!");
+    MOZ_RELEASE_ASSERT(aContentType < nsICacheEntry::CONTENT_TYPE_LAST);
+    return mCountByType[aContentType];
+  }
+
   uint32_t Dirty() {
     MOZ_ASSERT(!mStateLogged, "CacheIndexStats::Dirty() - state logged!");
     return mDirty;
@@ -566,6 +590,12 @@ class CacheIndexStats {
     return mSize;
   }
 
+  uint32_t SizeByType(uint8_t aContentType) {
+    MOZ_ASSERT(!mStateLogged, "CacheIndexStats::SizeByType() - state logged!");
+    MOZ_RELEASE_ASSERT(aContentType < nsICacheEntry::CONTENT_TYPE_LAST);
+    return mSizeByType[aContentType];
+  }
+
   void BeforeChange(const CacheIndexEntry* aEntry) {
 #ifdef DEBUG_STATS
     if (!mDisableLogging) {
@@ -582,7 +612,9 @@ class CacheIndexStats {
 #endif
     if (aEntry) {
       MOZ_ASSERT(mCount);
+      uint8_t contentType = aEntry->GetContentType();
       mCount--;
+      mCountByType[contentType]--;
       if (aEntry->IsDirty()) {
         MOZ_ASSERT(mDirty);
         mDirty--;
@@ -605,6 +637,7 @@ class CacheIndexStats {
           } else {
             MOZ_ASSERT(mSize >= aEntry->GetFileSize());
             mSize -= aEntry->GetFileSize();
+            mSizeByType[contentType] -= aEntry->GetFileSize();
           }
         }
       }
@@ -619,7 +652,9 @@ class CacheIndexStats {
     mStateLogged = false;
 #endif
     if (aEntry) {
+      uint8_t contentType = aEntry->GetContentType();
       ++mCount;
+      ++mCountByType[contentType];
       if (aEntry->IsDirty()) {
         mDirty++;
       }
@@ -636,6 +671,7 @@ class CacheIndexStats {
             mEmpty++;
           } else {
             mSize += aEntry->GetFileSize();
+            mSizeByType[contentType] += aEntry->GetFileSize();
           }
         }
       }
@@ -651,12 +687,14 @@ class CacheIndexStats {
 
  private:
   uint32_t mCount;
+  uint32_t mCountByType[nsICacheEntry::CONTENT_TYPE_LAST];
   uint32_t mNotInitialized;
   uint32_t mRemoved;
   uint32_t mDirty;
   uint32_t mFresh;
   uint32_t mEmpty;
   uint32_t mSize;
+  uint32_t mSizeByType[nsICacheEntry::CONTENT_TYPE_LAST];
 #ifdef DEBUG
   // We completely remove the data about an entry from the stats in
   // BeforeChange() and set this flag to true. The entry is then modified,
@@ -709,12 +747,13 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
   // MUST be initialized. Call to AddEntry() or EnsureEntryExists() and to
   // InitEntry() must precede the call to this method.
   // Pass nullptr if the value didn't change.
-  static nsresult UpdateEntry(
-      const SHA1Sum::Hash* aHash, const uint32_t* aFrecency,
-      const bool* aHasAltData, const uint16_t* aOnStartTime,
-      const uint16_t* aOnStopTime, const uint8_t* aContentType,
-      const uint16_t* aBaseDomainAccessCount, const uint32_t aTelemetryReportID,
-      const uint32_t* aSize);
+  static nsresult UpdateEntry(const SHA1Sum::Hash* aHash,
+                              const uint32_t* aFrecency,
+                              const bool* aHasAltData,
+                              const uint16_t* aOnStartTime,
+                              const uint16_t* aOnStopTime,
+                              const uint8_t* aContentType,
+                              const uint32_t* aSize);
 
   // Remove all entries from the index. Called when clearing the whole cache.
   static nsresult RemoveAll();
@@ -793,8 +832,9 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
   virtual ~CacheIndex();
 
   NS_IMETHOD OnFileOpened(CacheFileHandle* aHandle, nsresult aResult) override;
-  nsresult OnFileOpenedInternal(FileOpenHelper* aOpener,
-                                CacheFileHandle* aHandle, nsresult aResult);
+  void OnFileOpenedInternal(FileOpenHelper* aOpener, CacheFileHandle* aHandle,
+                            nsresult aResult,
+                            const StaticMutexAutoLock& aProofOfLock);
   NS_IMETHOD OnDataWritten(CacheFileHandle* aHandle, const char* aBuf,
                            nsresult aResult) override;
   NS_IMETHOD OnDataRead(CacheFileHandle* aHandle, char* aBuf,
@@ -803,7 +843,8 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
   NS_IMETHOD OnEOFSet(CacheFileHandle* aHandle, nsresult aResult) override;
   NS_IMETHOD OnFileRenamed(CacheFileHandle* aHandle, nsresult aResult) override;
 
-  nsresult InitInternal(nsIFile* aCacheDirectory);
+  nsresult InitInternal(nsIFile* aCacheDirectory,
+                        const StaticMutexAutoLock& aProofOfLock);
   void PreShutdownInternal();
 
   // This method returns false when index is not initialized or is shut down.
@@ -817,14 +858,16 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
                           OriginAttrsHash aOriginAttrsHash, bool aAnonymous);
 
   // Checks whether any of the information about the entry has changed.
-  static bool HasEntryChanged(
-      CacheIndexEntry* aEntry, const uint32_t* aFrecency,
-      const bool* aHasAltData, const uint16_t* aOnStartTime,
-      const uint16_t* aOnStopTime, const uint8_t* aContentType,
-      const uint16_t* aBaseDomainAccessCount, const uint32_t* aSize);
+  static bool HasEntryChanged(CacheIndexEntry* aEntry,
+                              const uint32_t* aFrecency,
+                              const bool* aHasAltData,
+                              const uint16_t* aOnStartTime,
+                              const uint16_t* aOnStopTime,
+                              const uint8_t* aContentType,
+                              const uint32_t* aSize);
 
   // Merge all pending operations from mPendingUpdates into mIndex.
-  void ProcessPendingOperations();
+  void ProcessPendingOperations(const StaticMutexAutoLock& aProofOfLock);
 
   // Following methods perform writing of the index file.
   //
@@ -836,14 +879,14 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
   //
   // Starts writing of index when both limits (minimal delay between writes and
   // minimum number of changes in index) were exceeded.
-  bool WriteIndexToDiskIfNeeded();
+  bool WriteIndexToDiskIfNeeded(const StaticMutexAutoLock& aProofOfLock);
   // Starts writing of index file.
-  void WriteIndexToDisk();
+  void WriteIndexToDisk(const StaticMutexAutoLock& aProofOfLock);
   // Serializes part of mIndex hashtable to the write buffer a writes the buffer
   // to the file.
-  void WriteRecords();
+  void WriteRecords(const StaticMutexAutoLock& aProofOfLock);
   // Finalizes writing process.
-  void FinishWrite(bool aSucceeded);
+  void FinishWrite(bool aSucceeded, const StaticMutexAutoLock& aProofOfLock);
 
   // Following methods perform writing of the journal during shutdown. All these
   // methods must be called only during shutdown since they write/delete files
@@ -853,7 +896,7 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
   // When the log is written successfully, the dirty flag in index file is
   // cleared.
   nsresult GetFile(const nsACString& aName, nsIFile** _retval);
-  nsresult RemoveFile(const nsACString& aName);
+  void RemoveFile(const nsACString& aName);
   void RemoveAllIndexFiles();
   void RemoveJournalAndTempFile();
   // Writes journal to the disk and clears dirty flag in index header.
@@ -896,17 +939,17 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
   // FF crashes during parsing of the index.
   //
   // Initiates reading index from disk.
-  void ReadIndexFromDisk();
+  void ReadIndexFromDisk(const StaticMutexAutoLock& aProofOfLock);
   // Starts reading data from index file.
-  void StartReadingIndex();
+  void StartReadingIndex(const StaticMutexAutoLock& aProofOfLock);
   // Parses data read from index file.
-  void ParseRecords();
+  void ParseRecords(const StaticMutexAutoLock& aProofOfLock);
   // Starts reading data from journal file.
-  void StartReadingJournal();
+  void StartReadingJournal(const StaticMutexAutoLock& aProofOfLock);
   // Parses data read from journal file.
-  void ParseJournal();
+  void ParseJournal(const StaticMutexAutoLock& aProofOfLock);
   // Merges entries from journal into mIndex.
-  void MergeJournal();
+  void MergeJournal(const StaticMutexAutoLock& aProofOfLock);
   // In debug build this method checks that we have no fresh entry in mIndex
   // after we finish reading index and before we process pending operations.
   void EnsureNoFreshEntry();
@@ -914,12 +957,12 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
   // to make sure mIndexStats contains correct information.
   void EnsureCorrectStats();
   // Finalizes reading process.
-  void FinishRead(bool aSucceeded);
+  void FinishRead(bool aSucceeded, const StaticMutexAutoLock& aProofOfLock);
 
   // Following methods perform updating and building of the index.
   // Timer callback that starts update or build process.
   static void DelayedUpdate(nsITimer* aTimer, void* aClosure);
-  void DelayedUpdateLocked();
+  void DelayedUpdateLocked(const StaticMutexAutoLock& aProofOfLock);
   // Posts timer event that start update or build process.
   nsresult ScheduleUpdateTimer(uint32_t aDelay);
   nsresult SetupDirectoryEnumerator();
@@ -930,20 +973,22 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
   bool IsUpdatePending();
   // Iterates through all files in entries directory that we didn't create/open
   // during this session, parses them and adds the entries to the index.
-  void BuildIndex();
+  void BuildIndex(const StaticMutexAutoLock& aProofOfLock);
 
-  bool StartUpdatingIndexIfNeeded(bool aSwitchingToReadyState = false);
+  bool StartUpdatingIndexIfNeeded(const StaticMutexAutoLock& aProofOfLock,
+                                  bool aSwitchingToReadyState = false);
   // Starts update or build process or fires a timer when it is too early after
   // startup.
-  void StartUpdatingIndex(bool aRebuild);
+  void StartUpdatingIndex(bool aRebuild,
+                          const StaticMutexAutoLock& aProofOfLock);
   // Iterates through all files in entries directory that we didn't create/open
   // during this session and theirs last modified time is newer than timestamp
   // in the index header. Parses the files and adds the entries to the index.
-  void UpdateIndex();
+  void UpdateIndex(const StaticMutexAutoLock& aProofOfLock);
   // Finalizes update or build process.
-  void FinishUpdate(bool aSucceeded);
+  void FinishUpdate(bool aSucceeded, const StaticMutexAutoLock& aProofOfLock);
 
-  void RemoveNonFreshEntries();
+  void RemoveNonFreshEntries(const StaticMutexAutoLock& aProofOfLock);
 
   enum EState {
     // Initial state in which the index is not usable
@@ -996,7 +1041,7 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
   };
 
   static char const* StateString(EState aState);
-  void ChangeState(EState aNewState);
+  void ChangeState(EState aNewState, const StaticMutexAutoLock& aProofOfLock);
   void NotifyAsyncGetDiskConsumptionCallbacks();
 
   // Allocates and releases buffer used for reading and writing index.
@@ -1004,19 +1049,21 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
   void ReleaseBuffer();
 
   // Methods used by CacheIndexEntryAutoManage to keep the iterators up to date.
-  void AddRecordToIterators(CacheIndexRecord* aRecord);
-  void RemoveRecordFromIterators(CacheIndexRecord* aRecord);
-  void ReplaceRecordInIterators(CacheIndexRecord* aOldRecord,
-                                CacheIndexRecord* aNewRecord);
+  void AddRecordToIterators(CacheIndexRecordWrapper* aRecord,
+                            const StaticMutexAutoLock& aProofOfLock);
+  void RemoveRecordFromIterators(CacheIndexRecordWrapper* aRecord,
+                                 const StaticMutexAutoLock& aProofOfLock);
+  void ReplaceRecordInIterators(CacheIndexRecordWrapper* aOldRecord,
+                                CacheIndexRecordWrapper* aNewRecord,
+                                const StaticMutexAutoLock& aProofOfLock);
 
   // Memory reporting (private part)
   size_t SizeOfExcludingThisInternal(mozilla::MallocSizeOf mallocSizeOf) const;
 
   void ReportHashStats();
 
-  // Reports telemetry about cache, i.e. size, entry count, content type stats
-  // and first party cache isolation stats. Clears first party cache isolation
-  // counters stored in the index entries and bumps a telemetry report ID.
+  // Reports telemetry about cache, i.e. size, entry count and content type
+  // stats.
   void DoTelemetryReport();
 
   static mozilla::StaticRefPtr<CacheIndex> gInstance;
@@ -1125,7 +1172,7 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
   class FrecencyArray {
     class Iterator {
      public:
-      explicit Iterator(nsTArray<CacheIndexRecord*>* aRecs)
+      explicit Iterator(nsTArray<RefPtr<CacheIndexRecordWrapper>>* aRecs)
           : mRecs(aRecs), mIdx(0) {
         while (!Done() && !(*mRecs)[mIdx]) {
           mIdx++;
@@ -1134,7 +1181,7 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
 
       bool Done() const { return mIdx == mRecs->Length(); }
 
-      CacheIndexRecord* Get() const {
+      CacheIndexRecordWrapper* Get() const {
         MOZ_ASSERT(!Done());
         return (*mRecs)[mIdx];
       }
@@ -1148,7 +1195,7 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
       }
 
      private:
-      nsTArray<CacheIndexRecord*>* mRecs;
+      nsTArray<RefPtr<CacheIndexRecordWrapper>>* mRecs;
       uint32_t mIdx;
     };
 
@@ -1158,19 +1205,22 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
     FrecencyArray() : mUnsortedElements(0), mRemovedElements(0) {}
 
     // Methods used by CacheIndexEntryAutoManage to keep the array up to date.
-    void AppendRecord(CacheIndexRecord* aRecord);
-    void RemoveRecord(CacheIndexRecord* aRecord);
-    void ReplaceRecord(CacheIndexRecord* aOldRecord,
-                       CacheIndexRecord* aNewRecord);
-    void SortIfNeeded();
+    void AppendRecord(CacheIndexRecordWrapper* aRecord,
+                      const StaticMutexAutoLock& aProofOfLock);
+    void RemoveRecord(CacheIndexRecordWrapper* aRecord,
+                      const StaticMutexAutoLock& aProofOfLock);
+    void ReplaceRecord(CacheIndexRecordWrapper* aOldRecord,
+                       CacheIndexRecordWrapper* aNewRecord,
+                       const StaticMutexAutoLock& aProofOfLock);
+    void SortIfNeeded(const StaticMutexAutoLock& aProofOfLock);
 
     size_t Length() const { return mRecs.Length() - mRemovedElements; }
-    void Clear() { mRecs.Clear(); }
+    void Clear(const StaticMutexAutoLock& aProofOfLock) { mRecs.Clear(); }
 
    private:
     friend class CacheIndex;
 
-    nsTArray<CacheIndexRecord*> mRecs;
+    nsTArray<RefPtr<CacheIndexRecordWrapper>> mRecs;
     uint32_t mUnsortedElements;
     // Instead of removing elements from the array immediately, we null them out
     // and the iterator skips them when accessing the array. The null pointers
@@ -1210,8 +1260,8 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
           mSize(0) {}
     virtual ~DiskConsumptionObserver() {
       if (mObserver && !NS_IsMainThread()) {
-        NS_ReleaseOnMainThreadSystemGroup("DiskConsumptionObserver::mObserver",
-                                          mObserver.forget());
+        NS_ReleaseOnMainThread("DiskConsumptionObserver::mObserver",
+                               mObserver.forget());
       }
     }
 
@@ -1235,7 +1285,7 @@ class CacheIndex final : public CacheFileIOListener, public nsIRunnable {
   };
 
   // List of async observers that want to get disk consumption information
-  nsTArray<RefPtr<DiskConsumptionObserver> > mDiskConsumptionObservers;
+  nsTArray<RefPtr<DiskConsumptionObserver>> mDiskConsumptionObservers;
 
   // Number of bytes written to the cache since the last telemetry report
   uint64_t mTotalBytesWritten;
