@@ -30,6 +30,9 @@ class nsIThread;
 class nsIEventTarget;
 
 namespace mozilla {
+
+class OriginAttributes;
+
 namespace net {
 
 class CacheStorageService;
@@ -302,12 +305,16 @@ class CacheStorageService final : public nsICacheStorageService,
                            bool aSkipSizeCheck, bool aPin, bool aReplace,
                            CacheEntryHandle** aResult);
 
+  nsresult ClearOriginInternal(
+      const nsAString& aOrigin,
+      const mozilla::OriginAttributes& aOriginAttributes, bool aAnonymous);
+
   static CacheStorageService* sSelf;
 
   mozilla::Mutex mLock;
   mozilla::Mutex mForcedValidEntriesLock;
 
-  bool mShutdown;
+  Atomic<bool, Relaxed> mShutdown;
 
   // Accessible only on the service thread
   class MemoryPool {
@@ -350,6 +357,13 @@ class CacheStorageService final : public nsICacheStorageService,
   }
 
   nsCOMPtr<nsITimer> mPurgeTimer;
+#ifdef MOZ_TSAN
+  // In OnMemoryConsumptionChange() we check whether the timer exists, but we
+  // cannot grab the lock there (see comment 6 in bug 1614637) and TSan reports
+  // a data race. This data race is harmless, so we use this atomic flag only in
+  // TSan build to suppress it.
+  Atomic<bool, Relaxed> mPurgeTimerActive;
+#endif
 
   class PurgeFromMemoryRunnable : public Runnable {
    public:
@@ -359,7 +373,7 @@ class CacheStorageService final : public nsICacheStorageService,
           mWhat(aWhat) {}
 
    private:
-    virtual ~PurgeFromMemoryRunnable() {}
+    virtual ~PurgeFromMemoryRunnable() = default;
 
     NS_IMETHOD Run() override;
 
@@ -383,7 +397,7 @@ class CacheStorageService final : public nsICacheStorageService,
     void Notify();
 
    private:
-    virtual ~IOThreadSuspender() {}
+    virtual ~IOThreadSuspender() = default;
     NS_IMETHOD Run() override;
 
     Monitor mMon;
