@@ -6,7 +6,6 @@
 #ifndef nsHttpChunkedDecoder_h__
 #define nsHttpChunkedDecoder_h__
 
-#include "nsAutoPtr.h"
 #include "nsError.h"
 #include "nsString.h"
 #include "nsHttpHeaderArray.h"
@@ -26,23 +25,23 @@ class nsHttpChunkedDecoder {
   bool ReachedEOF() { return mReachedEOF; }
 
   // called by the transaction to handle chunked content.
-  MOZ_MUST_USE nsresult HandleChunkedContent(char* buf, uint32_t count,
-                                             uint32_t* contentRead,
-                                             uint32_t* contentRemaining);
+  [[nodiscard]] nsresult HandleChunkedContent(char* buf, uint32_t count,
+                                              uint32_t* contentRead,
+                                              uint32_t* contentRemaining);
 
   nsHttpHeaderArray* Trailers() { return mTrailers.get(); }
 
-  nsHttpHeaderArray* TakeTrailers() { return mTrailers.forget(); }
+  UniquePtr<nsHttpHeaderArray> TakeTrailers() { return std::move(mTrailers); }
 
   // How mush data is still missing(needs to arrive) from the current chunk.
   uint32_t GetChunkRemaining() { return mChunkRemaining; }
 
  private:
-  MOZ_MUST_USE nsresult ParseChunkRemaining(char* buf, uint32_t count,
-                                            uint32_t* countRead);
+  [[nodiscard]] nsresult ParseChunkRemaining(char* buf, uint32_t count,
+                                             uint32_t* countRead);
 
  private:
-  nsAutoPtr<nsHttpHeaderArray> mTrailers;
+  UniquePtr<nsHttpHeaderArray> mTrailers;
   uint32_t mChunkRemaining;
   nsCString mLineBuf;  // may hold a partial line
   bool mReachedEOF;
