@@ -2,30 +2,77 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-var EXPORTED_SYMBOLS = [
-  "ErrorHandler",
-  "SyncScheduler",
-];
+var EXPORTED_SYMBOLS = ["ErrorHandler", "SyncScheduler"];
 
-ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
-ChromeUtils.import("resource://gre/modules/Services.jsm");
-ChromeUtils.import("resource://gre/modules/Log.jsm");
-ChromeUtils.import("resource://services-sync/constants.js");
-ChromeUtils.import("resource://services-sync/util.js");
-ChromeUtils.import("resource://services-common/logmanager.js");
-ChromeUtils.import("resource://services-common/async.js");
-ChromeUtils.import("resource://services-common/utils.js");
+const { XPCOMUtils } = ChromeUtils.import(
+  "resource://gre/modules/XPCOMUtils.jsm"
+);
+const { Services } = ChromeUtils.import("resource://gre/modules/Services.jsm");
+const { Log } = ChromeUtils.import("resource://gre/modules/Log.jsm");
+const {
+  CREDENTIALS_CHANGED,
+  ENGINE_APPLY_FAIL,
+  ENGINE_UNKNOWN_FAIL,
+  IDLE_OBSERVER_BACK_DELAY,
+  LOGIN_FAILED_INVALID_PASSPHRASE,
+  LOGIN_FAILED_LOGIN_REJECTED,
+  LOGIN_FAILED_NETWORK_ERROR,
+  LOGIN_FAILED_NO_PASSPHRASE,
+  LOGIN_SUCCEEDED,
+  MASTER_PASSWORD_LOCKED,
+  MASTER_PASSWORD_LOCKED_RETRY_INTERVAL,
+  MAX_ERROR_COUNT_BEFORE_BACKOFF,
+  MINIMUM_BACKOFF_INTERVAL,
+  MULTI_DEVICE_THRESHOLD,
+  NO_SYNC_NODE_FOUND,
+  NO_SYNC_NODE_INTERVAL,
+  OVER_QUOTA,
+  RESPONSE_OVER_QUOTA,
+  SCORE_UPDATE_DELAY,
+  SERVER_MAINTENANCE,
+  SINGLE_USER_THRESHOLD,
+  STATUS_OK,
+  SYNC_FAILED_PARTIAL,
+  SYNC_SUCCEEDED,
+  kSyncBackoffNotMet,
+  kSyncMasterPasswordLocked,
+} = ChromeUtils.import("resource://services-sync/constants.js");
+const { Svc, Utils } = ChromeUtils.import("resource://services-sync/util.js");
+const { LogManager } = ChromeUtils.import(
+  "resource://services-common/logmanager.js"
+);
+const { Async } = ChromeUtils.import("resource://services-common/async.js");
+const { CommonUtils } = ChromeUtils.import(
+  "resource://services-common/utils.js"
+);
 
-ChromeUtils.defineModuleGetter(this, "Status",
-                               "resource://services-sync/status.js");
-ChromeUtils.defineModuleGetter(this, "AddonManager",
-                               "resource://gre/modules/AddonManager.jsm");
-XPCOMUtils.defineLazyServiceGetter(this, "IdleService",
-                                   "@mozilla.org/widget/idleservice;1",
-                                   "nsIIdleService");
-XPCOMUtils.defineLazyServiceGetter(this, "CaptivePortalService",
-                                   "@mozilla.org/network/captive-portal-service;1",
-                                   "nsICaptivePortalService");
+ChromeUtils.defineModuleGetter(
+  this,
+  "Status",
+  "resource://services-sync/status.js"
+);
+ChromeUtils.defineModuleGetter(
+  this,
+  "AddonManager",
+  "resource://gre/modules/AddonManager.jsm"
+);
+ChromeUtils.defineModuleGetter(
+  this,
+  "fxAccounts",
+  "resource://gre/modules/FxAccounts.jsm"
+);
+XPCOMUtils.defineLazyServiceGetter(
+  this,
+  "IdleService",
+  "@mozilla.org/widget/idleservice;1",
+  "nsIIdleService"
+);
+XPCOMUtils.defineLazyServiceGetter(
+  this,
+  "CaptivePortalService",
+  "@mozilla.org/network/captive-portal-service;1",
+  "nsICaptivePortalService"
+);
 
 // Get the value for an interval that's stored in preferences. To save users
 // from themselves (and us from them!) the minimum time they can specify
@@ -41,9 +88,11 @@ function SyncScheduler(service) {
 SyncScheduler.prototype = {
   _log: Log.repository.getLogger("Sync.SyncScheduler"),
 
-  _fatalLoginStatus: [LOGIN_FAILED_NO_PASSPHRASE,
-                      LOGIN_FAILED_INVALID_PASSPHRASE,
-                      LOGIN_FAILED_LOGIN_REJECTED],
+  _fatalLoginStatus: [
+    LOGIN_FAILED_NO_PASSPHRASE,
+    LOGIN_FAILED_INVALID_PASSPHRASE,
+    LOGIN_FAILED_LOGIN_REJECTED,
+  ],
 
   /**
    * The nsITimer object that schedules the next sync. See scheduleNextSync().
@@ -53,10 +102,18 @@ SyncScheduler.prototype = {
   setDefaults: function setDefaults() {
     this._log.trace("Setting SyncScheduler policy values to defaults.");
 
-    this.singleDeviceInterval = getThrottledIntervalPreference("scheduler.fxa.singleDeviceInterval");
-    this.idleInterval         = getThrottledIntervalPreference("scheduler.idleInterval");
-    this.activeInterval       = getThrottledIntervalPreference("scheduler.activeInterval");
-    this.immediateInterval    = getThrottledIntervalPreference("scheduler.immediateInterval");
+    this.singleDeviceInterval = getThrottledIntervalPreference(
+      "scheduler.fxa.singleDeviceInterval"
+    );
+    this.idleInterval = getThrottledIntervalPreference(
+      "scheduler.idleInterval"
+    );
+    this.activeInterval = getThrottledIntervalPreference(
+      "scheduler.activeInterval"
+    );
+    this.immediateInterval = getThrottledIntervalPreference(
+      "scheduler.immediateInterval"
+    );
 
     // A user is non-idle on startup by default.
     this.idle = false;
@@ -81,6 +138,26 @@ SyncScheduler.prototype = {
     Svc.Prefs.set("nextSync", Math.floor(value / 1000));
   },
 
+  get missedFxACommandsFetchInterval() {
+    return Services.prefs.getIntPref(
+      "identity.fxaccounts.commands.missed.fetch_interval"
+    );
+  },
+
+  get missedFxACommandsLastFetch() {
+    return Services.prefs.getIntPref(
+      "identity.fxaccounts.commands.missed.last_fetch",
+      0
+    );
+  },
+
+  set missedFxACommandsLastFetch(val) {
+    Services.prefs.setIntPref(
+      "identity.fxaccounts.commands.missed.last_fetch",
+      val
+    );
+  },
+
   get syncInterval() {
     return this._syncInterval;
   },
@@ -127,8 +204,10 @@ SyncScheduler.prototype = {
     // We don't check on the NetworkLinkService however, because it gave us
     // false positives in the past in a vm environment.
     try {
-      if (Services.io.offline ||
-          CaptivePortalService.state == CaptivePortalService.LOCKED_PORTAL) {
+      if (
+        Services.io.offline ||
+        CaptivePortalService.state == CaptivePortalService.LOCKED_PORTAL
+      ) {
         return true;
       }
     } catch (ex) {
@@ -138,27 +217,53 @@ SyncScheduler.prototype = {
   },
 
   _initPrefGetters() {
-
-    XPCOMUtils.defineLazyPreferenceGetter(this,
-      "idleTime", "services.sync.scheduler.idleTime");
-    XPCOMUtils.defineLazyPreferenceGetter(this,
-      "maxResyncs", "services.sync.maxResyncs", 0);
+    XPCOMUtils.defineLazyPreferenceGetter(
+      this,
+      "idleTime",
+      "services.sync.scheduler.idleTime"
+    );
+    XPCOMUtils.defineLazyPreferenceGetter(
+      this,
+      "maxResyncs",
+      "services.sync.maxResyncs",
+      0
+    );
 
     // The number of clients we have is maintained in preferences via the
     // clients engine, and only updated after a successsful sync.
-    XPCOMUtils.defineLazyPreferenceGetter(this,
-      "numDesktopClients", "services.sync.clients.devices.desktop", 0);
-    XPCOMUtils.defineLazyPreferenceGetter(this,
-      "numMobileClients", "services.sync.clients.devices.mobile", 0);
+    XPCOMUtils.defineLazyPreferenceGetter(
+      this,
+      "numDesktopClients",
+      "services.sync.clients.devices.desktop",
+      0
+    );
+    XPCOMUtils.defineLazyPreferenceGetter(
+      this,
+      "numMobileClients",
+      "services.sync.clients.devices.mobile",
+      0
+    );
 
     // Scheduler state that seems to be read more often than it's written.
     // We also check if the value has changed before writing in the setters.
-    XPCOMUtils.defineLazyPreferenceGetter(this,
-      "_syncThreshold", "services.sync.syncThreshold", SINGLE_USER_THRESHOLD);
-    XPCOMUtils.defineLazyPreferenceGetter(this,
-      "_syncInterval", "services.sync.syncInterval", this.singleDeviceInterval);
-    XPCOMUtils.defineLazyPreferenceGetter(this,
-      "_globalScore", "services.sync.globalScore", 0);
+    XPCOMUtils.defineLazyPreferenceGetter(
+      this,
+      "_syncThreshold",
+      "services.sync.syncThreshold",
+      SINGLE_USER_THRESHOLD
+    );
+    XPCOMUtils.defineLazyPreferenceGetter(
+      this,
+      "_syncInterval",
+      "services.sync.syncInterval",
+      this.singleDeviceInterval
+    );
+    XPCOMUtils.defineLazyPreferenceGetter(
+      this,
+      "_globalScore",
+      "services.sync.globalScore",
+      0
+    );
   },
 
   init: function init() {
@@ -197,8 +302,12 @@ SyncScheduler.prototype = {
     switch (topic) {
       case "weave:engine:score:updated":
         if (Status.login == LOGIN_SUCCEEDED) {
-          CommonUtils.namedTimer(this.calculateScore, SCORE_UPDATE_DELAY, this,
-                           "_scoreTimer");
+          CommonUtils.namedTimer(
+            this.calculateScore,
+            SCORE_UPDATE_DELAY,
+            this,
+            "_scoreTimer"
+          );
         }
         break;
       case "network:link-status-changed":
@@ -209,13 +318,15 @@ SyncScheduler.prototype = {
         // if that's true, then the only downside is that we won't sync as
         // promptly).
         let isOffline = this.offline;
-        this._log.debug(`Network link status changed to "${data}". Offline?`,
-                        isOffline);
+        this._log.debug(
+          `Network link status changed to "${data}". Offline?`,
+          isOffline
+        );
         // Data may be one of `up`, `down`, `change`, or `unknown`. We only want
         // to sync if it's "up".
         if (data == "up" && !isOffline) {
           this._log.debug("Network link looks up. Syncing.");
-          this.scheduleNextSync(0, {why: topic});
+          this.scheduleNextSync(0, { why: topic });
         } else if (data == "down") {
           // Unschedule pending syncs if we know we're going down. We don't do
           // this via `checkSyncStatus`, since link status isn't reflected in
@@ -250,17 +361,24 @@ SyncScheduler.prototype = {
         }
 
         let sync_interval;
+        let nextSyncReason = "schedule";
         this.updateGlobalScore();
-        if (this.globalScore > 0 && Status.service == STATUS_OK) {
-          // The global score should be 0 after a sync. If it's not, items were
-          // changed during the last sync, and we should schedule an immediate
-          // follow-up sync.
+        if (
+          this.globalScore > this.syncThreshold &&
+          Status.service == STATUS_OK
+        ) {
+          // The global score should be 0 after a sync. If it's not, either
+          // items were changed during the last sync (and we should schedule an
+          // immediate follow-up sync), or an engine skipped
           this._resyncs++;
           if (this._resyncs <= this.maxResyncs) {
             sync_interval = 0;
+            nextSyncReason = "resync";
           } else {
-            this._log.warn(`Resync attempt ${this._resyncs} exceeded ` +
-                           `maximum ${this.maxResyncs}`);
+            this._log.warn(
+              `Resync attempt ${this._resyncs} exceeded ` +
+                `maximum ${this.maxResyncs}`
+            );
             Svc.Obs.notify("weave:service:resyncs-finished");
           }
         } else {
@@ -275,7 +393,7 @@ SyncScheduler.prototype = {
           this._log.trace("Scheduling a sync at interval NO_SYNC_NODE_FOUND.");
           sync_interval = NO_SYNC_NODE_INTERVAL;
         }
-        this.scheduleNextSync(sync_interval, {why: "schedule"});
+        this.scheduleNextSync(sync_interval, { why: nextSyncReason });
         break;
       case "weave:engine:sync:finish":
         if (data == "clients") {
@@ -297,7 +415,9 @@ SyncScheduler.prototype = {
           // Try again later, just as if we threw an error... only without the
           // error count.
           this._log.debug("Couldn't log in: master password is locked.");
-          this._log.trace("Scheduling a sync at MASTER_PASSWORD_LOCKED_RETRY_INTERVAL");
+          this._log.trace(
+            "Scheduling a sync at MASTER_PASSWORD_LOCKED_RETRY_INTERVAL"
+          );
           this.scheduleAtInterval(MASTER_PASSWORD_LOCKED_RETRY_INTERVAL);
         } else if (!this._fatalLoginStatus.includes(Status.login)) {
           // Not a fatal login error, just an intermittent network or server
@@ -321,7 +441,9 @@ SyncScheduler.prototype = {
       case "FxA:hawk:backoff:interval":
       case "weave:service:backoff:interval":
         let requested_interval = subject * 1000;
-        this._log.debug("Got backoff notification: " + requested_interval + "ms");
+        this._log.debug(
+          "Got backoff notification: " + requested_interval + "ms"
+        );
         // Leave up to 25% more time for the back off.
         let interval = requested_interval * (1 + Math.random() * 0.25);
         Status.backoffInterval = interval;
@@ -338,34 +460,37 @@ SyncScheduler.prototype = {
         break;
       case "weave:engine:sync:applied":
         let numItems = subject.succeeded;
-        this._log.trace("Engine " + data + " successfully applied " + numItems +
-                        " items.");
+        this._log.trace(
+          "Engine " + data + " successfully applied " + numItems + " items."
+        );
         if (numItems) {
           this.hasIncomingItems = true;
         }
         if (subject.newFailed) {
-          this._log.error(`Engine ${data} found ${subject.newFailed} new records that failed to apply`);
+          this._log.error(
+            `Engine ${data} found ${subject.newFailed} new records that failed to apply`
+          );
         }
         break;
       case "weave:service:setup-complete":
-         Services.prefs.savePrefFile(null);
-         IdleService.addIdleObserver(this, this.idleTime);
-         Svc.Obs.add("wake_notification", this);
-         Svc.Obs.add("captive-portal-login-success", this);
-         Svc.Obs.add("sleep_notification", this);
-         break;
+        Services.prefs.savePrefFile(null);
+        IdleService.addIdleObserver(this, this.idleTime);
+        Svc.Obs.add("wake_notification", this);
+        Svc.Obs.add("captive-portal-login-success", this);
+        Svc.Obs.add("sleep_notification", this);
+        break;
       case "weave:service:start-over":
-         this.setDefaults();
-         try {
-           IdleService.removeIdleObserver(this, this.idleTime);
-         } catch (ex) {
-           if (ex.result != Cr.NS_ERROR_FAILURE) {
-             throw ex;
-           }
-           // In all likelihood we didn't have an idle observer registered yet.
-           // It's all good.
-         }
-         break;
+        this.setDefaults();
+        try {
+          IdleService.removeIdleObserver(this, this.idleTime);
+        } catch (ex) {
+          if (ex.result != Cr.NS_ERROR_FAILURE) {
+            throw ex;
+          }
+          // In all likelihood we didn't have an idle observer registered yet.
+          // It's all good.
+        }
+        break;
       case "idle":
         this._log.trace("We're idle.");
         this.idle = true;
@@ -377,19 +502,26 @@ SyncScheduler.prototype = {
       case "active":
         this._log.trace("Received notification that we're back from idle.");
         this.idle = false;
-        CommonUtils.namedTimer(function onBack() {
-          if (this.idle) {
-            this._log.trace("... and we're idle again. " +
-                            "Ignoring spurious back notification.");
-            return;
-          }
+        CommonUtils.namedTimer(
+          function onBack() {
+            if (this.idle) {
+              this._log.trace(
+                "... and we're idle again. " +
+                  "Ignoring spurious back notification."
+              );
+              return;
+            }
 
-          this._log.trace("Genuine return from idle. Syncing.");
-          // Trigger a sync if we have multiple clients.
-          if (this.numClients > 1) {
-            this.scheduleNextSync(0, {why: topic});
-          }
-        }, IDLE_OBSERVER_BACK_DELAY, this, "idleDebouncerTimer");
+            this._log.trace("Genuine return from idle. Syncing.");
+            // Trigger a sync if we have multiple clients.
+            if (this.numClients > 1) {
+              this.scheduleNextSync(0, { why: topic });
+            }
+          },
+          IDLE_OBSERVER_BACK_DELAY,
+          this,
+          "idleDebouncerTimer"
+        );
         break;
       case "wake_notification":
         this._log.debug("Woke from sleep.");
@@ -400,7 +532,7 @@ SyncScheduler.prototype = {
           if (this.numClients > 1) {
             if (!this.offline) {
               this._log.debug("Online, will sync in 2s.");
-              this.scheduleNextSync(2000, {why: topic});
+              this.scheduleNextSync(2000, { why: topic });
             }
           }
         });
@@ -408,13 +540,13 @@ SyncScheduler.prototype = {
       case "captive-portal-login-success":
         this._log.debug("Captive portal login success. Scheduling a sync.");
         CommonUtils.nextTick(() => {
-          this.scheduleNextSync(3000, {why: topic});
+          this.scheduleNextSync(3000, { why: topic });
         });
         break;
       case "sleep_notification":
         if (this.service.engineManager.get("tabs")._tracker.modified) {
           this._log.debug("Going to sleep, doing a quick sync.");
-          this.scheduleNextSync(0, {engines: ["tabs"], why: "sleep"});
+          this.scheduleNextSync(0, { engines: ["tabs"], why: "sleep" });
         }
         break;
     }
@@ -446,9 +578,11 @@ SyncScheduler.prototype = {
   },
 
   updateGlobalScore() {
-    let engines = [this.service.clientsEngine].concat(this.service.engineManager.getEnabled());
+    let engines = [this.service.clientsEngine].concat(
+      this.service.engineManager.getEnabled()
+    );
     let globalScore = this.globalScore;
-    for (let i = 0;i < engines.length;i++) {
+    for (let i = 0; i < engines.length; i++) {
       this._log.trace(engines[i].name + ": score: " + engines[i].score);
       globalScore += engines[i].score;
       engines[i]._tracker.resetScore();
@@ -468,10 +602,13 @@ SyncScheduler.prototype = {
   updateClientMode: function updateClientMode() {
     // Nothing to do if it's the same amount
     let numClients = this.numClients;
-    if (numClients == this.numClientsLastSync)
+    if (numClients == this.numClientsLastSync) {
       return;
+    }
 
-    this._log.debug(`Client count: ${this.numClientsLastSync} -> ${numClients}`);
+    this._log.debug(
+      `Client count: ${this.numClientsLastSync} -> ${numClients}`
+    );
     this.numClientsLastSync = numClients;
 
     if (numClients <= 1) {
@@ -492,7 +629,7 @@ SyncScheduler.prototype = {
     // if we're in backoff, we'll schedule the next sync.
     let ignore = [kSyncBackoffNotMet, kSyncMasterPasswordLocked];
     let skip = this.service._checkSync(ignore);
-    this._log.trace("_checkSync returned \"" + skip + "\".");
+    this._log.trace('_checkSync returned "' + skip + '".');
     if (skip) {
       this.clearSyncTriggers();
       return;
@@ -506,7 +643,7 @@ SyncScheduler.prototype = {
       wait = 0;
       why = "score";
     }
-    this.scheduleNextSync(wait, {why});
+    this.scheduleNextSync(wait, { why });
   },
 
   /**
@@ -516,12 +653,13 @@ SyncScheduler.prototype = {
    */
   syncIfMPUnlocked(engines, why) {
     // No point if we got kicked out by the master password dialog.
-    if (Status.login == MASTER_PASSWORD_LOCKED &&
-        Utils.mpLocked()) {
+    if (Status.login == MASTER_PASSWORD_LOCKED && Utils.mpLocked()) {
       this._log.debug("Not initiating sync: Login status is " + Status.login);
 
       // If we're not syncing now, we need to schedule the next one.
-      this._log.trace("Scheduling a sync at MASTER_PASSWORD_LOCKED_RETRY_INTERVAL");
+      this._log.trace(
+        "Scheduling a sync at MASTER_PASSWORD_LOCKED_RETRY_INTERVAL"
+      );
       this.scheduleAtInterval(MASTER_PASSWORD_LOCKED_RETRY_INTERVAL);
       return;
     }
@@ -531,14 +669,31 @@ SyncScheduler.prototype = {
       return;
     }
     Services.tm.dispatchToMainThread(() => {
-      this.service.sync({engines, why});
+      this.service.sync({ engines, why });
+      const now = Math.round(new Date().getTime() / 1000);
+      // Only fetch missed messages in a "scheduled" sync so we don't race against
+      // the Push service reconnecting on a network link change for example.
+      if (
+        why == "schedule" &&
+        now >=
+          this.missedFxACommandsLastFetch + this.missedFxACommandsFetchInterval
+      ) {
+        fxAccounts.commands
+          .pollDeviceCommands()
+          .then(() => {
+            this.missedFxACommandsLastFetch = now;
+          })
+          .catch(e => {
+            this._log.error("Fetching missed remote commands failed.", e);
+          });
+      }
     });
   },
 
   /**
    * Set a timer for the next sync
    */
-  scheduleNextSync(interval, {engines = null, why = null} = {}) {
+  scheduleNextSync(interval, { engines = null, why = null } = {}) {
     // If no interval was specified, use the current sync interval.
     if (interval == null) {
       interval = this.syncInterval;
@@ -546,10 +701,14 @@ SyncScheduler.prototype = {
 
     // Ensure the interval is set to no less than the backoff.
     if (Status.backoffInterval && interval < Status.backoffInterval) {
-      this._log.trace("Requested interval " + interval +
-                      " ms is smaller than the backoff interval. " +
-                      "Using backoff interval " +
-                      Status.backoffInterval + " ms instead.");
+      this._log.trace(
+        "Requested interval " +
+          interval +
+          " ms is smaller than the backoff interval. " +
+          "Using backoff interval " +
+          Status.backoffInterval +
+          " ms instead."
+      );
       interval = Status.backoffInterval;
     }
     let nextSync = this.nextSync;
@@ -557,11 +716,13 @@ SyncScheduler.prototype = {
       // There's already a sync scheduled. Don't reschedule if there's already
       // a timer scheduled for sooner than requested.
       let currentInterval = nextSync - Date.now();
-      this._log.trace("There's already a sync scheduled in " +
-                      currentInterval + " ms.");
+      this._log.trace(
+        "There's already a sync scheduled in " + currentInterval + " ms."
+      );
       if (currentInterval < interval && this.syncTimer) {
-        this._log.trace("Ignoring scheduling request for next sync in " +
-                        interval + " ms.");
+        this._log.trace(
+          "Ignoring scheduling request for next sync in " + interval + " ms."
+        );
         return;
       }
     }
@@ -574,42 +735,55 @@ SyncScheduler.prototype = {
     }
 
     this._log.debug(`Next sync in ${interval} ms. (why=${why})`);
-    CommonUtils.namedTimer(() => { this.syncIfMPUnlocked(engines, why); },
-                           interval, this, "syncTimer");
+    CommonUtils.namedTimer(
+      () => {
+        this.syncIfMPUnlocked(engines, why);
+      },
+      interval,
+      this,
+      "syncTimer"
+    );
 
     // Save the next sync time in-case sync is disabled (logout/offline/etc.)
     this.nextSync = Date.now() + interval;
   },
 
-
   /**
    * Incorporates the backoff/retry logic used in error handling and elective
    * non-syncing.
    */
   scheduleAtInterval: function scheduleAtInterval(minimumInterval) {
-    let interval = Utils.calculateBackoff(this._syncErrors,
-                                          MINIMUM_BACKOFF_INTERVAL,
-                                          Status.backoffInterval);
+    let interval = Utils.calculateBackoff(
+      this._syncErrors,
+      MINIMUM_BACKOFF_INTERVAL,
+      Status.backoffInterval
+    );
     if (minimumInterval) {
       interval = Math.max(minimumInterval, interval);
     }
 
-    this._log.debug("Starting client-initiated backoff. Next sync in " +
-                    interval + " ms.");
-    this.scheduleNextSync(interval, {why: "client-backoff-schedule"});
+    this._log.debug(
+      "Starting client-initiated backoff. Next sync in " + interval + " ms."
+    );
+    this.scheduleNextSync(interval, { why: "client-backoff-schedule" });
   },
 
- /**
-  * Automatically start syncing after the given delay (in seconds).
-  *
-  * Applications can define the `services.sync.autoconnectDelay` preference
-  * to have this called automatically during start-up with the pref value as
-  * the argument. Alternatively, they can call it themselves to control when
-  * Sync should first start to sync.
-  */
+  /**
+   * Automatically start syncing after the given delay (in seconds).
+   *
+   * Applications can define the `services.sync.autoconnectDelay` preference
+   * to have this called automatically during start-up with the pref value as
+   * the argument. Alternatively, they can call it themselves to control when
+   * Sync should first start to sync.
+   */
   delayedAutoConnect: function delayedAutoConnect(delay) {
     if (this.service._checkSetup() == STATUS_OK) {
-      CommonUtils.namedTimer(this.autoConnect, delay * 1000, this, "_autoTimer");
+      CommonUtils.namedTimer(
+        this.autoConnect,
+        delay * 1000,
+        this,
+        "_autoTimer"
+      );
     }
   },
 
@@ -618,7 +792,7 @@ SyncScheduler.prototype = {
       // Schedule a sync based on when a previous sync was scheduled.
       // scheduleNextSync() will do the right thing if that time lies in
       // the past.
-      this.scheduleNextSync(this.nextSync - Date.now(), {why: "startup"});
+      this.scheduleNextSync(this.nextSync - Date.now(), { why: "startup" });
     }
 
     // Once autoConnect is called we no longer need _autoTimer.
@@ -639,18 +813,20 @@ SyncScheduler.prototype = {
     // backoff due to 5xx errors.
     if (!Status.enforceBackoff) {
       if (this._syncErrors < MAX_ERROR_COUNT_BEFORE_BACKOFF) {
-        this.scheduleNextSync(null, {why: "reschedule"});
+        this.scheduleNextSync(null, { why: "reschedule" });
         return;
       }
-      this._log.debug("Sync error count has exceeded " +
-                      MAX_ERROR_COUNT_BEFORE_BACKOFF + "; enforcing backoff.");
+      this._log.debug(
+        "Sync error count has exceeded " +
+          MAX_ERROR_COUNT_BEFORE_BACKOFF +
+          "; enforcing backoff."
+      );
       Status.enforceBackoff = true;
     }
 
     this.scheduleAtInterval();
   },
 
-
   /**
    * Remove any timers/observers that might trigger a sync
    */
@@ -659,10 +835,10 @@ SyncScheduler.prototype = {
     this.globalScore = this.nextSync = 0;
 
     // Clear out any scheduled syncs
-    if (this.syncTimer)
+    if (this.syncTimer) {
       this.syncTimer.clear();
+    }
   },
-
 };
 
 function ErrorHandler(service) {
@@ -684,14 +860,22 @@ ErrorHandler.prototype = {
   initLogs: function initLogs() {
     // Set the root Sync logger level based on a pref. All other logs will
     // inherit this level unless they specifically override it.
-    Log.repository.getLogger("Sync").manageLevelFromPref(`services.sync.log.logger`);
+    Log.repository
+      .getLogger("Sync")
+      .manageLevelFromPref(`services.sync.log.logger`);
     // And allow our specific log to have a custom level via a pref.
     this._log = Log.repository.getLogger("Sync.ErrorHandler");
     this._log.manageLevelFromPref("services.sync.log.logger.service.main");
 
-    let logs = ["Sync", "Services.Common", "FirefoxAccounts", "Hawk",
-                "browserwindow.syncui", "BookmarkSyncUtils", "addons.xpi",
-               ];
+    let logs = [
+      "Sync",
+      "Services.Common",
+      "FirefoxAccounts",
+      "Hawk",
+      "browserwindow.syncui",
+      "BookmarkSyncUtils",
+      "addons.xpi",
+    ];
 
     this._logManager = new LogManager(Svc.Prefs, logs, "sync");
   },
@@ -714,13 +898,17 @@ ErrorHandler.prototype = {
 
         this.checkServerError(exception);
 
-        Status.engines = [engine_name, exception.failureCode || ENGINE_UNKNOWN_FAIL];
+        Status.engines = [
+          engine_name,
+          exception.failureCode || ENGINE_UNKNOWN_FAIL,
+        ];
         if (Async.isShutdownException(exception)) {
-          this._log.debug(engine_name + " was interrupted due to the application shutting down");
+          this._log.debug(
+            engine_name +
+              " was interrupted due to the application shutting down"
+          );
         } else {
           this._log.debug(engine_name + " failed", exception);
-          Services.telemetry.getKeyedHistogramById("WEAVE_ENGINE_SYNC_ERRORS")
-                            .add(engine_name);
         }
         break;
       }
@@ -737,7 +925,9 @@ ErrorHandler.prototype = {
         if (Async.isShutdownException(exception)) {
           // If we are shutting down we just log the fact, attempt to flush
           // the log file and get out of here!
-          this._log.error("Sync was interrupted due to the application shutting down");
+          this._log.error(
+            "Sync was interrupted due to the application shutting down"
+          );
           this.resetFileLog();
           break;
         }
@@ -754,8 +944,7 @@ ErrorHandler.prototype = {
         // engine, Status.service will be SYNC_FAILED_PARTIAL despite
         // Status.sync being SYNC_SUCCEEDED.
         // *facepalm*
-        if (Status.sync == SYNC_SUCCEEDED &&
-            Status.service == STATUS_OK) {
+        if (Status.sync == SYNC_SUCCEEDED && Status.service == STATUS_OK) {
           // Great. Let's clear our mid-sync 401 note.
           this._log.trace("Clearing lastSyncReassigned.");
           Svc.Prefs.reset("lastSyncReassigned");
@@ -768,7 +957,22 @@ ErrorHandler.prototype = {
         break;
       case "weave:service:start-over:finish":
         // ensure we capture any logs between the last sync and the reset completing.
-        this.resetFileLog();
+        this.resetFileLog()
+          .then(() => {
+            // although for privacy reasons we also delete all logs (but we allow
+            // a preference to avoid this to help with debugging.)
+            if (!Svc.Prefs.get("log.keepLogsOnReset", false)) {
+              return this._logManager.removeAllLogs().then(() => {
+                Svc.Obs.notify("weave:service:remove-file-log");
+              });
+            }
+            return null;
+          })
+          .catch(err => {
+            // So we failed to delete the logs - take the ironic option of
+            // writing this error to the logs we failed to delete!
+            this._log.error("Failed to delete logs on reset", err);
+          });
         break;
     }
   },
@@ -784,9 +988,9 @@ ErrorHandler.prototype = {
     }
 
     let relevantAddons = addons.filter(x => x.isActive && !x.hidden);
-    this._log.debug("Addons installed", relevantAddons.length);
+    this._log.trace("Addons installed", relevantAddons.length);
     for (let addon of relevantAddons) {
-      this._log.debug(" - ${name}, version ${version}, id ${id}", addon);
+      this._log.trace(" - ${name}, version ${version}, id ${id}", addon);
     }
   },
 
@@ -801,7 +1005,9 @@ ErrorHandler.prototype = {
     }
     const logType = await this._logManager.resetFileLog();
     if (logType == this._logManager.ERROR_LOG_WRITTEN) {
-      Cu.reportError("Sync encountered an error - see about:sync-log for the log file.");
+      Cu.reportError(
+        "Sync encountered an error - see about:sync-log for the log file."
+      );
     }
     Svc.Obs.notify("weave:service:reset-file-log");
   },
@@ -840,7 +1046,7 @@ ErrorHandler.prototype = {
           Svc.Prefs.set("lastSyncReassigned", true);
         }
         this._log.info("Attempting to schedule another sync.");
-        this.service.scheduler.scheduleNextSync(delay, {why: "reschedule"});
+        this.service.scheduler.scheduleNextSync(delay, { why: "reschedule" });
         break;
 
       case 500:
@@ -856,8 +1062,10 @@ ErrorHandler.prototype = {
           } else {
             Status.login = SERVER_MAINTENANCE;
           }
-          Svc.Obs.notify("weave:service:backoff:interval",
-                         parseInt(retryAfter, 10));
+          Svc.Obs.notify(
+            "weave:service:backoff:interval",
+            parseInt(retryAfter, 10)
+          );
         }
         break;
     }
